@@ -58,8 +58,9 @@
  x		         Position	   Position	    0
  y		         Position	   Position    	    0
  layout			 Layout		   Layout	    NULL
- debug			 Boolean	   Boolean	    False
- resize			 Boolean	   Boolean	    True ###jp### 11/94
+ debug			 Debug		   Boolean	    False
+ resizeWidth		 Resize		   Boolean	    True ###jp### 11/94
+ resizeHeight		 Resize		   Boolean	    True ###jp### 11/94
  maximumWidth	         MaximumWidth	   Dimension	    0    ###jp### 11/94
  maximumHeight		 MaximumHeight	   Dimension	    0    ###jp### 11/94
  minimumWidth	         MinimumWidth	   Dimension	    0    ###jp### 11/94
@@ -72,7 +73,7 @@
 
  Name                    Class             RepType          Default Value
  ----                    -----             -------          -------------
- resizable               Boolean           Boolean          True
+ resizable               Resizable         Boolean          True
 
 */
   
@@ -81,10 +82,14 @@
 #define XtCLayout		"Layout"
 #define XtRLayout		"Layout"
 #define XtNdebug		"debug"
+#define XtCDebug		"Debug"
 #define XtNresizable		"resizable"
+#define XtCResizable		"Resizable"
 #define XtNresizeWidth		"resizeWidth"
+#ifndef XtCResize
+#   define XtCResize		"Resize"
+#endif
 #define XtNresizeHeight		"resizeHeight"
-#define XtNconditionedResize	"conditionedResize"
 #define XtNmaximumWidth		"maximumWidth"
 #define XtCMaximumWidth		"MaximumWidth"
 #define XtNmaximumHeight	"maximumHeight"
@@ -93,6 +98,8 @@
 #define XtCMinimumWidth		"MinimumWidth"
 #define XtNminimumHeight	"minimumHeight"
 #define XtCMinimumHeight	"MinimumHeight"
+#define XtNforcedWidth		"forcedWidth"
+#define XtNforcedHeight		"forcedHeight"
 
 /* Class record constant */
 
