@@ -2,7 +2,7 @@
 **
 ** FileSel.h
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -31,15 +31,16 @@
 
 #include INC_X11(Xfuncproto.h)
 
-#define XawFileSelectionRescan 	1
-#define XawFileSelectionFilter 	2
-#define XawFileSelectionDefaultScan 	3
-#define XawFileSelectionDone 		4
-#define XawFileSelectionCancel 	5
+#define XawFileSelectionRescan 	        (1<<0)
+#define XawFileSelectionFilter 	        (1<<1)
+#define XawFileSelectionDefaultScan 	(1<<2)
+#define XawFileSelectionDone 		(1<<3)
+#define XawFileSelectionCancel 	        (1<<4)
 
 /* New Fields */
 
 #define XtNreverseScrolling "reverseScrolling"
+#define XtCReverseScrolling "ReverseScrolling"
 
 #define XtNtmpDir "tmpDir"
 #define XtCTmpDir "TmpDir"
@@ -50,9 +51,11 @@
 #define XtNfilter "filter"
 #define XtCFilter "Filter"
 
-#define XtNviewMode "viewMode"
-#define XtCViewMode "ViewMode"
-#define XtRViewMode "ViewMode"
+#define XtNfilters "filters"
+#define XtCFilters "Filters"
+
+#define XtNdirs "dirs"
+#define XtCDirs "Dirs"
 
 #define XtNhighlightPixel "highlightPixel"
 #define XtCHighlightPixel "HighlightPixel"
