@@ -2,7 +2,7 @@
 **
 ** FileSelP.h
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -56,12 +56,14 @@ typedef struct _FileSelectionClassPart {
 
 typedef struct {
     /* resources */
-    Pixel	highlightPixel;
+    Pixel	highlight_pixel;
     String      path;
     String      filter;
-    int		view_mode;
+    String      filters;
+    String      dirs;
     int         buttons;
-    int		preferredButton;    
+    int		preferred_button;    
+    Boolean     reverse_scrolling;
     /* private data */
     Widget	pathframeFS;
     Widget	pathFS;
@@ -69,18 +71,22 @@ typedef struct {
     Widget	filterFS;
     Widget	old_textfieldFS;
     Widget	rescanbuttonFS;
-    Widget	filterbuttonFS;
-    Widget	homebuttonFS;
-    Widget	tmpbuttonFS;
+    Widget	filtersbuttonFS;
+    Widget	filtersmenuFS;
+    Widget	dirsmenuFS;
+    Widget	dirsbuttonFS;
     Widget	panedFS;
+    Widget	topscrollFS;
     Widget	topframeFS;
     Widget	topclipFS;
     Widget	topaaaFS;
     Widget	toplistFS;
+    Widget	curscrollFS;
     Widget	curframeFS;
     Widget	curclipFS;
     Widget	curaaaFS;
     Widget	curlistFS;
+    Widget	subscrollFS;
     Widget	subframeFS;
     Widget	subclipFS;
     Widget	subaaaFS;
@@ -96,15 +102,15 @@ typedef struct {
     FSDir       subdir;
     int         subdir_alloc;
     String      app_dir;
-    String      both;
-    String      current_dir;
+    String      current_path;
+    char        *current_dir;
+    char        *current_file;
     String      tmp_dir;
     String      path_field_value;
     String      filter_field_value;    
     int         internal_buttons;    
     unsigned long multiclick;
-    Pixel	old_highlightPixel;
-    Boolean     reverse_scrolling;
+    Pixel	old_highlight_pixel;
 } FileSelectionPart;
 
 typedef struct _FileSelectionClassRec {
