@@ -15,6 +15,7 @@
 
 /*
 #define MESSAGES
+#define MESSAGES1
 */
 #include "message.h"
 
@@ -136,7 +137,7 @@ WidgetClass frameWidgetClass =	(WidgetClass) (&frameClassRec);
  ****************************************************************/
 
 /*---------------------------------------------------*/
-/* cvtStringToFrameType */
+/* FrameConvertStringToFrameType */
 /*---------------------------------------------------*/
 
 #define done(type, value)  {			\
@@ -152,12 +153,12 @@ WidgetClass frameWidgetClass =	(WidgetClass) (&frameClassRec);
       to->addr = (XtPointer)&static_val;	\
    }						\
    to->size = sizeof(type);			\
-   ENDMESSAGE(cvtStringToFrameType)		\
+   ENDMESSAGE1(FrameConvertStringToFrameType)	\
    return True;					\
 }
 
-static  Boolean
-cvtStringToFrameType ( display, args, num_args,  from, to, converter_data)
+Boolean
+FrameConvertStringToFrameType ( display, args, num_args,  from, to, converter_data)
    Display *display;
    XrmValuePtr args;
    Cardinal *num_args;
@@ -167,7 +168,7 @@ cvtStringToFrameType ( display, args, num_args,  from, to, converter_data)
 {
    String s = (String) from->addr;
 
-   BEGINMESSAGE(cvtStringToFrameType)
+   BEGINMESSAGE1(FrameConvertStringToFrameType)
    if (XmuCompareISOLatin1(s, "raised")   == 0) done(XawFrameType, XawRAISED);
    if (XmuCompareISOLatin1(s, "sunken")   == 0) done(XawFrameType, XawSUNKEN);
    if (XmuCompareISOLatin1(s, "chiseled") == 0) done(XawFrameType, XawCHISELED);
@@ -186,7 +187,7 @@ ClassInitialize()
 {
    BEGINMESSAGE(ClassInitialize)
    XawInitializeWidgetSet();
-   XtSetTypeConverter(XtRString, XtRFrameType, cvtStringToFrameType,
+   XtSetTypeConverter(XtRString, XtRFrameType, FrameConvertStringToFrameType,
 		     (XtConvertArgList)NULL, 0, XtCacheNone, NULL);
    ENDMESSAGE(ClassInitialize)
 }
@@ -195,7 +196,7 @@ ClassInitialize()
 /* Initialize */
 /*---------------------------------------------------*/
 
-GC
+static GC
 shadow_getGC (w,pixel)
     Widget w;
     Pixel  pixel;
@@ -321,43 +322,46 @@ Resize(w)
   bottom_polygon[i].x = (short) (xx);	\
   bottom_polygon[i].y = (short) (yy)
 
-static void 
-DrawFrame (gw, x, y, w, h, frame_type, t, lightgc, darkgc)
-   Widget       gw;
-   Position     x;
-   Position     y;
-   Dimension    w; 
-   Dimension    h;
+void 
+FrameDrawFrame (gw, x, y, w, h, frame_type, fw, lightgc, darkgc)
+   Widget  gw;
+   int     x;
+   int     y;
+   int     w; 
+   int     h;
    XawFrameType frame_type;
-   Dimension    t;
-   GC           lightgc;
-   GC           darkgc;
+   int     fw;
+   GC      lightgc;
+   GC      darkgc;
 {
    XPoint top_polygon[6];
    XPoint bottom_polygon[6];
   
-   BEGINMESSAGE(DrawFrame)
+   BEGINMESSAGE1(FrameDrawFrame)
 
    if (lightgc == (GC)NULL ){
-      XtWarning("DrawFrame: lightgc is NULL.");
+      XtWarning("FrameDrawFrame: lightgc is NULL.");
+      ENDMESSAGE1(FrameDrawFrame)
       return;
    }
    if (darkgc == (GC)NULL ){
-      XtWarning("DrawFrame: darkgc is NULL.");
+      XtWarning("FrameDrawFrame: darkgc is NULL.");
+      ENDMESSAGE1(FrameDrawFrame)
       return;
    }
    if (!XtIsRealized(gw)) {
-      XtWarning("DrawFrame: widget is not realized.");
+      XtWarning("FrameDrawFrame: widget is not realized.");
+      ENDMESSAGE1(FrameDrawFrame)
       return;
    }
   
    if (frame_type == XawRAISED || frame_type == XawSUNKEN ) {
-      topPolygon (0,x    ,y    ); bottomPolygon (0,x+w  ,y+h  ); 
-      topPolygon (1,x+w  ,y    ); bottomPolygon (1,x    ,y+h  );
-      topPolygon (2,x+w-t,y+t  ); bottomPolygon (2,x+t  ,y+h-t);
-      topPolygon (3,x+t  ,y+t  ); bottomPolygon (3,x+w-t,y+h-t);
-      topPolygon (4,x+t  ,y+h-t); bottomPolygon (4,x+w-t,y+t  );
-      topPolygon (5,x    ,y+h  ); bottomPolygon (5,x+w  ,y    );
+      topPolygon (0,x    ,y    ); bottomPolygon (0,x+w  ,y+h    ); 
+      topPolygon (1,x+w  ,y    ); bottomPolygon (1,x    ,y+h    );
+      topPolygon (2,x+w-fw,y+fw); bottomPolygon (2,x+fw  ,y+h-fw);
+      topPolygon (3,x+fw,y+fw  ); bottomPolygon (3,x+w-fw,y+h-fw);
+      topPolygon (4,x+fw,y+h-fw); bottomPolygon (4,x+w-fw,y+fw  );
+      topPolygon (5,x    ,y+h  ); bottomPolygon (5,x+w  ,y      );
       if (frame_type == XawSUNKEN) {
          XFillPolygon(XtDisplayOfObject(gw), XtWindowOfObject(gw), darkgc,
 		      top_polygon, 6, Nonconvex, CoordModeOrigin);
@@ -370,38 +374,35 @@ DrawFrame (gw, x, y, w, h, frame_type, t, lightgc, darkgc)
 		      bottom_polygon, 6, Nonconvex, CoordModeOrigin);
       }
    }
-   else if ( frame_type == XawFrameMassiveRaised) {
-     if (t>=3) {
-        Dimension it,mt,ot;
+   else if (frame_type == XawFrameMassiveRaised) {
+     if (fw>=3) {
+        int it,mt,ot;
         ot = 1;
         it = 1;
-        mt = t-ot-it;
-        DrawFrame(gw, x, y, w, h, XawRAISED, ot, lightgc, darkgc);
-        DrawFrame(gw,(Position)(x+mt+ot), (Position)(y+mt+ot),
-		  (Dimension)(w-2*mt-2*ot), (Dimension)(h-2*mt-2*ot),
-		  XawSUNKEN, it, lightgc, darkgc);
+        mt = fw-ot-it;
+        FrameDrawFrame(gw, x, y, w, h, XawRAISED, ot, lightgc, darkgc);
+        FrameDrawFrame(gw,x+mt+ot,y+mt+ot,w-2*mt-2*ot,h-2*mt-2*ot,
+		       XawSUNKEN, it, lightgc, darkgc);
      }
    }
    else if ( frame_type == XawLEDGED ) {
-     Dimension it,ot;
-     it = ot = t/2;
-     if (t&1) it += 1;
-     DrawFrame(gw, x, y, w, h, XawRAISED, ot, lightgc, darkgc);
-     DrawFrame(gw,(Position)(x+ot), (Position)(y+ot),
-		  (Dimension)(w-2*ot), (Dimension)(h-2*ot),
+     int it,ot;
+     it = ot = fw/2;
+     if (fw&1) it += 1;
+     FrameDrawFrame(gw, x, y, w, h, XawRAISED, ot, lightgc, darkgc);
+     FrameDrawFrame(gw,x+ot, y+ot,w-2*ot, h-2*ot,
 		  XawSUNKEN, it, lightgc, darkgc);
    }
    else if ( frame_type == XawCHISELED ) {
-     Dimension it,ot;
-     it = ot = t/2;
-     if (t&1) it += 1;
-     DrawFrame(gw, x, y, w, h, XawSUNKEN, ot, lightgc, darkgc);
-     DrawFrame(gw,(Position)(x+ot),(Position)(y+ot),
-		  (Dimension)(w-2*ot), (Dimension)(h-2*ot),
-		  XawRAISED, it, lightgc, darkgc);
+     int it,ot;
+     it = ot = fw/2;
+     if (fw&1) it += 1;
+     FrameDrawFrame(gw, x, y, w, h, XawSUNKEN, ot, lightgc, darkgc);
+     FrameDrawFrame(gw,x+ot,y+ot,w-2*ot, h-2*ot,
+		    XawRAISED, it, lightgc, darkgc);
    }
 
-   ENDMESSAGE(DrawFrame)
+   ENDMESSAGE1(FrameDrawFrame)
 
 }
 #undef topPolygon
@@ -416,26 +417,20 @@ Redisplay(w, event, region)
    FrameWidget fw = (FrameWidget) w;
    int wh,ww,sw,bw;
 
-   BEGINMESSAGE(Redisplay)
+   BEGINMESSAGE1(Redisplay)
 
    bw = (int)FW_CORE.border_width;
    ww = (int)FW_CORE.width;
    wh = (int)FW_CORE.height;
    sw = (int)FW_SHADOW; 
    if (sw == 0 || 2*sw>ww || 2*sw>wh) {
-      INFMESSAGE(not enough space to display anything) ENDMESSAGE(Redisplay)
+      INFMESSAGE(not enough space to display anything) ENDMESSAGE1(Redisplay)
       return;
    }
-   DrawFrame(w,
-	     (Position)0,
-	     (Position)0,
-	     (Dimension)ww,
-	     (Dimension)wh,
-	     FW_FRAME.frame_type,
-	     (Dimension)sw,
-	     FW_FRAME.top_shadow_GC,
-	     FW_FRAME.bot_shadow_GC);
-   ENDMESSAGE(Redisplay)
+   FrameDrawFrame(w,0,0,ww,wh,FW_FRAME.frame_type,sw,
+		  FW_FRAME.top_shadow_GC,
+		  FW_FRAME.bot_shadow_GC);
+   ENDMESSAGE1(Redisplay)
 }
 
 /*---------------------------------------------------*/
@@ -585,8 +580,10 @@ QueryGeometry(w,request,preferred_return)
 {
    FrameWidget fw = (FrameWidget)w;
    Dimension nw,nh;
-   BEGINMESSAGE(QueryGeometry)
 
+   BEGINMESSAGE(QueryGeometry)
+   if (fw->composite.children && fw->composite.children[0])
+     GetDesiredSizeOfChild(fw->composite.children[0]);
    GetNaturalSize(fw,&nw,&nh);
    preferred_return->request_mode = (CWWidth|CWHeight);
    preferred_return->width  = nw;
