@@ -59,6 +59,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 
+#include "types.h"
 #include "d_memdebug.h"
 #include "main_resources.h"
 #include "main_globals.h"
@@ -108,9 +109,6 @@ extern int errno;
 
 #define ALLOW_PDF /*###jp### 02/29/96 */
 
-static void ComputeXdpi();
-static void ComputeYdpi();
-
 #define CURSOR_RESET   1
 #define CURSOR_BUSY    2
 #define CURSOR_SCROLL  3
@@ -161,10 +159,6 @@ static XtResource resources[] = {
 	  offset(urx), XtRImmediate, (XtPointer)612 },
     { XtNury, XtCBoundingBox, XtRInt, sizeof(int),
 	  offset(ury), XtRImmediate, (XtPointer)792 },
-    { XtNxdpi, XtCResolution, XtRFloat, sizeof(float),
-	  offset(xdpi), XtRCallProc, (XtPointer)ComputeXdpi },
-    { XtNydpi, XtCResolution, XtRFloat, sizeof(float),
-	  offset(ydpi), XtRCallProc, (XtPointer)ComputeYdpi },
     { XtNlxdpi, XtCLResolution, XtRInt, sizeof(long),
 	  offset(lxdpi), XtRImmediate, (XtPointer)0 },
     { XtNlydpi, XtCLResolution, XtRInt, sizeof(long),
@@ -173,12 +167,14 @@ static XtResource resources[] = {
        offset(pref_width), XtRImmediate, (XtPointer)1},
     {XtNpreferredHeight, XtCPreferredHeight, XtRDimension, sizeof(Dimension),
        offset(pref_height), XtRImmediate, (XtPointer)1},
-    {XtNautoSetup,XtCBoolean, XtRBoolean, sizeof(Boolean),
-       offset(auto_setup), XtRImmediate, (XtPointer)True},
-    { XtNsafer, XtCSafer, XtRBoolean, sizeof(Boolean),
+    {XtNsafer, XtCSafer, XtRBoolean, sizeof(Boolean),
           offset(safer), XtRImmediate, (XtPointer)True },
-    { XtNinterpreter, XtCInterpreter, XtRString, sizeof(String),
+    {XtNinterpreter, XtCInterpreter, XtRString, sizeof(String),
           offset(interpreter), XtRString, "gs" },
+#if 0
+    {XtNhighlightPixel, XtCHighlightPixel, XtRPixel, sizeof(Pixel),
+	offset(highlight_pixel), XtRString, XtDefaultForeground},
+#endif
 #undef offset
 };
 
@@ -213,15 +209,6 @@ static XtActionsRec actions[] =
     {"cursor",	action_changeCursor},
 };
 
-/* notify takes zero to four parameters.  The first two give the width and
- * height of the zoom requested in the default user coordinate system.
- * If they are omitted, a default value of 72 is provided.  If the second
- * parameter is omitted, the zoom area is assumed to be a square.
- * The next two parameters give the desired resolution of the zoom window.
- * If they are omitted, a default value of 300 is provided. If the four
- * parameter is omitted, the y resolution is assumed to be equal to the
- * x resolution.
-*/
 static char translations[] =
 "<Message>:message()\n\
 ";
@@ -272,42 +259,6 @@ GhostviewClassRec ghostviewClassRec = {
 
 WidgetClass ghostviewWidgetClass = (WidgetClass)&ghostviewClassRec;
 
-/* Procedures that compute the default xdpi and ydpi from display parameters */
-
-/*###################################################################################*/
-/*  ComputeXdpi  */
-/*###################################################################################*/
-
-static void
-ComputeXdpi(w, offset, value)
-    Widget w;
-    int offset;
-    XrmValue *value;
-{
-    static float xdpi;
-    BEGINMESSAGE(ComputeXdpi)
-    xdpi = 25.4 * WidthOfScreen(XtScreen(w)) / WidthMMOfScreen(XtScreen(w));
-    value->addr = (XtPointer) &xdpi;
-    ENDMESSAGE(ComputeXdpi)
-}
-
-/*###################################################################################*/
-/*  ComputeYdpi */
-/*###################################################################################*/
-
-static void
-ComputeYdpi(w, offset, value)
-    Widget w;
-    int offset;
-    XrmValue *value;
-{
-    static float ydpi;
-    BEGINMESSAGE(ComputeYdpi)
-    ydpi = 25.4 * HeightOfScreen(XtScreen(w)) / HeightMMOfScreen(XtScreen(w));
-    value->addr = (XtPointer) &ydpi;
-    ENDMESSAGE(ComputeYdpi)
-}
-
 /*###################################################################################*/
 /* Message action routine.
  * Passes ghostscript message events back to application via
@@ -407,98 +358,38 @@ ChangeCursor(gvw, which)
 /*###################################################################################*/
 /* Notify action routine.
  * Calculates where the user clicked in the default user coordinate system.
- * Call the callbacks with the point of click and size of zoom window
- * requested.
+ * Call the callbacks with the point of click.
  */
 /*###################################################################################*/
 
 static void
 Notify(w, event, params, num_params)
-    Widget w;
-    XEvent *event;
-    String *params;
-    Cardinal *num_params;
+  Widget w;
+  XEvent *event;
+  String *params;
+  Cardinal *num_params;
 {
-    GhostviewWidget gvw = (GhostviewWidget) w;
-    GhostviewReturnStruct ret_val;
-
-    BEGINMESSAGE1(Notify)
+  GhostviewWidget gvw = (GhostviewWidget) w;
+  GhostviewReturnStruct ret_val;
+  int psx,psy;
 
+  BEGINMESSAGE1(Notify)
+  GhostviewCoordsXtoPS(w,event->xbutton.x,event->xbutton.y,&psx,&psy);
 #if 0
-    /* notify takes zero to four parameters.  The first two give the width and
-     * height of the zoom requested in the default user coordinate system.
-     * If they are omitted, a default value of 72 is provided.  If the second
-     * parameter is omitted, the zoom area is assumed to be a square.
-     * The next two parameters give the desired resolution of the zoom window.
-     * If they are omitted, a default value of 300 is provided. If the four
-     * parameter is omitted, the y resolution is assumed to be equal to the
-     * x resolution.
-     */
-
-    switch (*num_params) {
-    case 0:
-	ret_val.width = ret_val.height = 72;
-	ret_val.xdpi = ret_val.ydpi = 300;
-	break;
-    case 1:
-	ret_val.width = ret_val.height = atoi(params[0]);
-	ret_val.xdpi = ret_val.ydpi = 300;
-	break;
-    case 2:
-	ret_val.width = atoi(params[0]);
-	ret_val.height = atoi(params[1]);
-	ret_val.xdpi = ret_val.ydpi = 300;
-	break;
-    case 3:
-	ret_val.width = atoi(params[0]);
-	ret_val.height = atoi(params[1]);
-	ret_val.xdpi = ret_val.ydpi = atoi(params[2]);
-	break;
-    default:
-	ret_val.width = atoi(params[0]);
-	ret_val.height = atoi(params[1]);
-	ret_val.xdpi = atoi(params[2]);
-	ret_val.ydpi = atoi(params[3]);
-	break;
-    }
+  {
+    int wx,wy;
+    GhostviewCoordsPStoX(w,psx,psy,&wx,&wy);
+    printf("evx=%d evy=%d\n",event->xbutton.x,event->xbutton.y);
+    printf("psx=%d psy=%d\n",psx,psy);
+    printf("wx =%d wy =%d\n\n",wx,wy);
+  }
 #endif
-
-    ret_val.width  = 0;
-    ret_val.height = 0;
-
-    IIMESSAGE1(event->xbutton.x,event->xbutton.y)
-    switch (gvw->ghostview.orientation) {
-    case XtPageOrientationPortrait:
-	ret_val.psx = gvw->ghostview.llx + 
-		      event->xbutton.x * 72.0 / gvw->ghostview.xdpi;
-	ret_val.psy = gvw->ghostview.ury - 
-		      event->xbutton.y * 72.0 / gvw->ghostview.ydpi;
-	break;
-    case XtPageOrientationLandscape:
-	ret_val.psx = gvw->ghostview.llx + 
-		      event->xbutton.y * 72.0 / gvw->ghostview.ydpi;
-	ret_val.psy = gvw->ghostview.lly + 
-		      event->xbutton.x * 72.0 / gvw->ghostview.xdpi;
-	break;
-    case XtPageOrientationUpsideDown:
-	ret_val.psx = gvw->ghostview.urx - 
-		      event->xbutton.x * 72.0 / gvw->ghostview.xdpi;
-	ret_val.psy = gvw->ghostview.lly + 
-		      event->xbutton.y * 72.0 / gvw->ghostview.ydpi;
-	break;
-    case XtPageOrientationSeascape:
-	ret_val.psx = gvw->ghostview.urx - 
-		      event->xbutton.y * 72.0 / gvw->ghostview.ydpi;
-	ret_val.psy = gvw->ghostview.ury - 
-		      event->xbutton.x * 72.0 / gvw->ghostview.xdpi;
-	break;
-    default:
-	ret_val.psx = 13;
-	ret_val.psy = 13;
-        break;
-    }
-    XtCallCallbackList(w, gvw->ghostview.callback, (XtPointer) &ret_val);
-    ENDMESSAGE1(Notify)
+  ret_val.width  = 0;
+  ret_val.height = 0;
+  ret_val.psx = psx;
+  ret_val.psy = psy;
+  XtCallCallbackList(w, gvw->ghostview.callback, (XtPointer) &ret_val);
+  ENDMESSAGE1(Notify)
 }
 
 #ifndef SEEK_SET
@@ -618,7 +509,7 @@ Input(client_data, source, id)
                by hand. This we do since gs doesn't seem to finish a page properly if
                the file's records are terminated by <cr>. (###jp### 02.06.96)
 */
-               IMESSAGE(bytes_written)
+               IMESSAGE1(bytes_written)
                if (gvw->ghostview.bytes_left == 0 &&
                    gvw->ghostview.buffer_bytes_left == 0 &&
                    bytes_written > 0 &&
@@ -674,12 +565,14 @@ Output(client_data, source, id)
     if (*source == gvw->ghostview.interpreter_output) {
 	bytes = read(gvw->ghostview.interpreter_output, buf, GV_BUFSIZ);
 	if (bytes == 0) { /* EOF occurred */
+	    INFMESSAGE(EOF occured in interpreter_output)
 	    close(gvw->ghostview.interpreter_output);
 	    gvw->ghostview.interpreter_output = -1;
 	    XtRemoveInput(gvw->ghostview.interpreter_output_id);
             ENDMESSAGE(Output)
 	    return;
 	} else if (bytes == -1) {
+	    INFMESSAGE(something bad happened in interpreter_output)
 	    InterpreterFailed(w);		/* Something bad happened */
             ENDMESSAGE(Output)
 	    return;
@@ -687,12 +580,14 @@ Output(client_data, source, id)
     } else if (*source == gvw->ghostview.interpreter_error) {
 	bytes = read(gvw->ghostview.interpreter_error, buf, GV_BUFSIZ);
 	if (bytes == 0) { /* EOF occurred */
+	    INFMESSAGE(EOF occurred in interpreter_error)
 	    close(gvw->ghostview.interpreter_error);
 	    gvw->ghostview.interpreter_error = -1;
 	    XtRemoveInput(gvw->ghostview.interpreter_error_id);
             ENDMESSAGE(Output)
 	    return;
 	} else if (bytes == -1) {
+	    INFMESSAGE(something bad happened in interpreter_error)
 	    InterpreterFailed(w);		/* Something bad happened */
             ENDMESSAGE(Output)
 	    return;
@@ -763,9 +658,20 @@ Initialize(request, new, args, num_args)
     GhostviewWidget rgvw = (GhostviewWidget) request;
 
     BEGINMESSAGE(Initialize)
+
     values.foreground = new->core.background_pixel;
-    mask = GCForeground;
+    values.background = new->core.background_pixel;
+    values.graphics_exposures = False;
+    mask = GCGraphicsExposures|GCForeground|GCBackground;
     ngvw->ghostview.gc = XtGetGC(new, mask, &values);
+    values.foreground   = new->core.background_pixel;
+    values.background   = new->core.background_pixel;
+    values.line_width   = 1;
+    values.function     = GXxor;
+    values.graphics_exposures = False;
+    mask=(GCFunction|GCGraphicsExposures|GCBackground|GCForeground|GCLineWidth);
+    ngvw->ghostview.highlight_gc = XtGetGC(new,mask,&values); 
+
     ngvw->ghostview.mwin = None;
     ngvw->ghostview.disable_start = False;
     ngvw->ghostview.interpreter_pid = -1;
@@ -831,8 +737,6 @@ Realize(w, valueMask, attributes)
     XtCreateWindow(w,(unsigned int)InputOutput,(Visual *)CopyFromParent,*valueMask,attributes);
     IMESSAGE(gvw->core.width) IMESSAGE(gvw->core.height) 
 
-    if (gvw->ghostview.auto_setup) Setup(w);
-
     ENDMESSAGE(Realize)
 }
 
@@ -849,6 +753,7 @@ Destroy(w)
     BEGINMESSAGE(Destroy)
     StopInterpreter(w);
     XtReleaseGC(w, gvw->ghostview.gc);
+    XtReleaseGC(w, gvw->ghostview.highlight_gc);
     if (gvw->ghostview.input_buffer) GV_XtFree(gvw->ghostview.input_buffer);
 #ifdef VMS
     if (gvw->ghostview.output_buffer) GV_XtFree(gvw->ghostview.output_buffer);
@@ -900,35 +805,15 @@ SetBackground(w,clear)
 }
 
 /*###################################################################################*/
-/* Process resize request.  Requested size cannot be changed.
- * NOTE: This routine may be called before the widget is realized.
- * (It was a surprise to me.)
- * If the widget is realized, start a new interpreter by calling Setup().
- * If setup() actually started a new interpreter and it is taking input
- * from stdin, send a refresh message to the application.  This is the
- * only way that the application can be notified that it needs to resend
- * the input because someone forced a new window size on the widget.
- */
+/* Resize */
 /*###################################################################################*/
 
 static void
 Resize(w)
-    Widget w;
+  Widget w;
 {
-   GhostviewWidget gvw = (GhostviewWidget) w;
-
-   BEGINMESSAGE(Resize)
-   if (gvw->ghostview.auto_setup) {
-      Layout(w, False, False);
-      if (!XtIsRealized(w)) {INFMESSAGE(not realized) ENDMESSAGE(Resize) return;}
-      INFIIMESSAGE(resizing to:, w->core.width,w->core.height)
-      if (Setup(w)) {
-   	 if (gvw->ghostview.filename == NULL) {
-	    XtCallCallbackList(w, gvw->ghostview.message_callback, "Refresh");
-	 }
-      }
-   }
-   ENDMESSAGE(Resize)
+  BEGINMESSAGE(Resize)
+  ENDMESSAGE(Resize)
 }
 
 /*###################################################################################*/
@@ -984,7 +869,9 @@ SetValues(current, request, new)
 
 	XtReleaseGC(current, cgvw->ghostview.gc);
 	values.foreground = new->core.background_pixel;
-	mask = GCForeground;
+	values.background = new->core.background_pixel;
+	values.graphics_exposures = False;
+	mask = GCGraphicsExposures|GCForeground|GCBackground;
 	ngvw->ghostview.gc = XtGetGC(new, mask, &values);
     }
     IIMESSAGE(ngvw->core.width,ngvw->core.height)
@@ -1143,143 +1030,67 @@ Layout(w, xfree, yfree)
 
 static Boolean
 ComputeSize(w, xfree, yfree, width, height)
-    Widget w;
-    Boolean xfree, yfree;	/* Am I allowed to change width or height */
-    Dimension *width, *height;
+  Widget w;
+  Boolean xfree, yfree;	/* Am I allowed to change width or height */
+  Dimension *width, *height;
 {
-    GhostviewWidget gvw = (GhostviewWidget) w;
-    Dimension new_width = *width;
-    Dimension new_height = *height;
-    float newxdpi, newydpi;
-    Boolean change;
+  GhostviewWidget gvw = (GhostviewWidget) w;
+  Dimension new_width = *width;
+  Dimension new_height = *height;
+  float newxdpi=72, newydpi=72,xdpi,ydpi;
+  Boolean change;
+  int dpsw,dpsh;
 
-    BEGINMESSAGE(ComputeSize)
-    if (xfree && yfree) {
-        INFMESSAGE(x and y are free)
-	/* width and height can be changed, calculate window size according */
-	/* to xpdi and ydpi */
-	switch (gvw->ghostview.orientation) {
-	case XtPageOrientationPortrait:
-	case XtPageOrientationUpsideDown:
-	    new_width = (Dimension)(
-                ((float)(gvw->ghostview.urx-gvw->ghostview.llx+1)
-                *gvw->ghostview.xdpi)/72.0+0.5
-            );
-	    new_height = (Dimension)(
-                ((float)(gvw->ghostview.ury-gvw->ghostview.lly+1)
-                *gvw->ghostview.ydpi)/72.0+0.5
-            );
-	    break;
-	case XtPageOrientationLandscape:
-	case XtPageOrientationSeascape:
-	    new_width = (Dimension)(
-                ((float)(gvw->ghostview.ury-gvw->ghostview.lly+1)
-                *gvw->ghostview.xdpi)/72.0+0.5
-            );
-	    new_height =(Dimension)(
-                ((float)(gvw->ghostview.urx-gvw->ghostview.llx+1)
-                *gvw->ghostview.ydpi)/72.0+0.5
-            ); 
-	    break;
-	case XtPageOrientationUnspecified:
-	  /* should not be reached */
-            break;
-	}
-    } else if (xfree) {
-        INFMESSAGE(x is free)
-	/* height is fixed.  Preserve aspect ratio by recomputing */
-	/* ydpi and xdpi */
-	switch (gvw->ghostview.orientation) {
-	case XtPageOrientationPortrait:
-	case XtPageOrientationUpsideDown:
-	    newydpi = gvw->core.height * 72.0 /
-		      (gvw->ghostview.ury - gvw->ghostview.lly+1);
-	    newxdpi = newydpi * gvw->ghostview.xdpi / gvw->ghostview.ydpi;
-	    gvw->ghostview.xdpi = newxdpi;
-	    gvw->ghostview.ydpi = newydpi;
-	    new_width = (Dimension)(
-                ((float)(gvw->ghostview.urx-gvw->ghostview.llx+1)
-                *gvw->ghostview.xdpi)/72.0+0.5
-            );
-	    break;
-	case XtPageOrientationLandscape:
-	case XtPageOrientationSeascape:
-	    newydpi = gvw->core.height * 72.0 /
-		      (gvw->ghostview.urx - gvw->ghostview.llx+1);
-	    newxdpi = newydpi * gvw->ghostview.xdpi / gvw->ghostview.ydpi;
-	    gvw->ghostview.xdpi = newxdpi;
-	    gvw->ghostview.ydpi = newydpi;
-	    new_width = (Dimension)(
-                ((float)(gvw->ghostview.ury-gvw->ghostview.lly+1)
-                *gvw->ghostview.xdpi)/72.0+0.5
-            );
-	    break;
-	case XtPageOrientationUnspecified:
-            break;
-	}
+  BEGINMESSAGE(ComputeSize)
+
+  switch (gvw->ghostview.orientation) {
+    case XtPageOrientationLandscape:
+    case XtPageOrientationSeascape:
+      dpsw = gvw->ghostview.ury-gvw->ghostview.lly+1;
+      dpsh = gvw->ghostview.urx-gvw->ghostview.llx+1;
+      xdpi = gvw->ghostview.ydpi;
+      ydpi = gvw->ghostview.xdpi;
+      break;
+    case XtPageOrientationPortrait:
+    case XtPageOrientationUpsideDown:
+    default:
+      dpsw = gvw->ghostview.urx-gvw->ghostview.llx+1;
+      dpsh = gvw->ghostview.ury-gvw->ghostview.lly+1;
+      xdpi = gvw->ghostview.xdpi;
+      ydpi = gvw->ghostview.ydpi;
+      break;
+  }
+  if (dpsw<1) dpsw = 1;
+  if (dpsh<1) dpsh = 1;
+  xdpi = gvw->ghostview.xdpi;
+  ydpi = gvw->ghostview.ydpi;
+  if (xfree && yfree) {
+    INFMESSAGE(x and y are free)
+    new_width  = (((float)dpsw*72.0)/xdpi + 0.5);
+    new_height = (((float)dpsh*72.0)/ydpi + 0.5);
+  } else {
+    if (!xfree) newxdpi = (float)dpsw * 72.0 / (float)gvw->core.width;
+    if (!yfree) newydpi = (float)dpsh * 72.0 / (float)gvw->core.height;
+    if (xfree) {
+      INFMESSAGE(x is free)
+      newxdpi = newydpi * xdpi / ydpi;
+      new_width = (((float)dpsw*72.0)/newxdpi+0.5);
     } else if (yfree) {
-         INFMESSAGE(y is free)
-	/* width is fixed.  Preserve aspect ratio by recomputing */
-	/* xdpi and ydpi */
-	switch (gvw->ghostview.orientation) {
-	case XtPageOrientationPortrait:
-	case XtPageOrientationUpsideDown:
-	    newxdpi = gvw->core.width * 72.0 /
-		      (gvw->ghostview.urx - gvw->ghostview.llx+1);
-	    newydpi = newxdpi * gvw->ghostview.ydpi / gvw->ghostview.xdpi;
-	    gvw->ghostview.xdpi = newxdpi;
-	    gvw->ghostview.ydpi = newydpi;
-	    new_height = (Dimension)(
-                ((float)(gvw->ghostview.ury-gvw->ghostview.lly+1)
-                *gvw->ghostview.ydpi)/72.0+0.5
-            );
-	    break;
-	case XtPageOrientationLandscape:
-	case XtPageOrientationSeascape:
-	    newxdpi = gvw->core.width * 72.0 /
-		      (gvw->ghostview.ury - gvw->ghostview.lly+1);
-	    newydpi = newxdpi * gvw->ghostview.ydpi / gvw->ghostview.xdpi;
-	    gvw->ghostview.xdpi = newxdpi;
-	    gvw->ghostview.ydpi = newydpi;
-	    new_height =(Dimension)(
-                ((float)(gvw->ghostview.urx-gvw->ghostview.llx+1)
-                *gvw->ghostview.ydpi)/72.0+0.5
-            );
-	    break;
-	case XtPageOrientationUnspecified:
-	  /* should not be reached */
-            break;
-	}
-    } else {
-        INFMESSAGE(x and y are not free)
-	/* height and width are fixed.  Just have to live with it. */
-	switch (gvw->ghostview.orientation) {
-	case XtPageOrientationPortrait:
-	case XtPageOrientationUpsideDown:
-	    gvw->ghostview.xdpi = gvw->core.width * 72.0 /
-				  (gvw->ghostview.urx - gvw->ghostview.llx+1);
-	    gvw->ghostview.ydpi = gvw->core.height * 72.0 /
-				  (gvw->ghostview.ury - gvw->ghostview.lly+1);
-	    break;
-	case XtPageOrientationLandscape:
-	case XtPageOrientationSeascape:
-	    gvw->ghostview.xdpi = gvw->core.width * 72.0 /
-				  (gvw->ghostview.ury - gvw->ghostview.lly+1);
-	    gvw->ghostview.ydpi = gvw->core.height * 72.0 /
-				  (gvw->ghostview.urx - gvw->ghostview.llx+1);
-	    break;
-	case XtPageOrientationUnspecified:
-	  /* should not be reached */
-            break;
-	}
+      INFMESSAGE(y is free)
+      newydpi = newxdpi * ydpi / xdpi;
+      new_height = (((float)(dpsh)*72.0)/newydpi+0.5);
     }
-    FMESSAGE(gvw->ghostview.xdpi)
-    FMESSAGE(gvw->ghostview.ydpi)
-    change = (new_width != *width) || (new_height != *height);
-    *width = new_width; 
-    *height = new_height;
-    gvw->ghostview.pref_width  = new_width; 
-    gvw->ghostview.pref_height = new_height;
+    gvw->ghostview.xdpi = newxdpi;
+    gvw->ghostview.ydpi = newydpi;
+  }
+
+  FMESSAGE(gvw->ghostview.xdpi)
+  FMESSAGE(gvw->ghostview.ydpi)
+  change = (new_width != *width) || (new_height != *height);
+  *width = new_width; 
+  *height = new_height;
+  gvw->ghostview.pref_width  = new_width; 
+  gvw->ghostview.pref_height = new_height;
 #if 0
     printf("--------------\n");
     if (xfree) printf(" x is free\n");
@@ -1333,6 +1144,7 @@ Setup(w)
    GhostviewWidgetClass gvc = (GhostviewWidgetClass) XtClass(w);
    char buf[GV_BUFSIZ];
    Pixmap bpixmap;
+   float xdpi,ydpi;
 
    BEGINMESSAGE(Setup)
 
@@ -1401,12 +1213,33 @@ Setup(w)
 
    gvw->ghostview.gs_width = gvw->core.width;
    gvw->ghostview.gs_height = gvw->core.height;
+
+   { 
+     int dpsw,dpsh;
+     switch (gvw->ghostview.orientation) {
+       case XtPageOrientationLandscape:
+       case XtPageOrientationSeascape:
+	 dpsw = gvw->ghostview.ury-gvw->ghostview.lly+1;
+	 dpsh = gvw->ghostview.urx-gvw->ghostview.llx+1;
+	 break;
+       case XtPageOrientationPortrait:
+       case XtPageOrientationUpsideDown:
+       default:
+	 dpsw = gvw->ghostview.urx-gvw->ghostview.llx+1;
+	 dpsh = gvw->ghostview.ury-gvw->ghostview.lly+1;
+	 break;
+     }
+     if (dpsw<1) dpsw=1; if (dpsh<1) dpsh = 1;
+     xdpi =  (float)gvw->core.width * 72.0 / (float)dpsw;
+     ydpi =  (float)gvw->core.height* 72.0 / (float)dpsh;
+  }
+
    IIMESSAGE(gvw->ghostview.gs_width,gvw->ghostview.gs_height)
    sprintf(buf, "%ld %d %d %d %d %d %g %g %d %d %d %d",
 	   bpixmap, gvw->ghostview.orientation,
 	   (int)gvw->ghostview.llx, (int)gvw->ghostview.lly,
 	   gvw->ghostview.urx, gvw->ghostview.ury,
-	   gvw->ghostview.xdpi, gvw->ghostview.ydpi,
+	   xdpi, ydpi,
 	   gvw->ghostview.left_margin, gvw->ghostview.bottom_margin,
 	   gvw->ghostview.right_margin, gvw->ghostview.top_margin);
 
@@ -1432,7 +1265,6 @@ Setup(w)
 
    gvw->ghostview.background_cleared = 0;
    SetBackground(w,True);
-   if (gvw->ghostview.auto_setup) StartInterpreter(w);
 
    ENDMESSAGE(Setup)
    return True;
@@ -1450,6 +1282,7 @@ Setup(w)
  * non-blocking I/O so that the user interface never "hangs" because of
  * a write to ghostscript.
  */
+
 static void
 StartInterpreter(w)
     Widget w;
@@ -1468,6 +1301,8 @@ StartInterpreter(w)
     int argc = 0;
     int ret;
 
+    BEGINMESSAGE(StartInterpreter)
+
     StopInterpreter(w);
 
     /* Clear the window before starting a new interpreter. */
@@ -1549,10 +1384,10 @@ StartInterpreter(w)
     gvw->ghostview.changed = False;
     gvw->ghostview.busy = True;
     ChangeCursor(gvw,CURSOR_BUSY);
-#if defined(SYSV) || defined(USG) || defined(linux) /* for linux ###jp###*/
-#define vfork fork
+#if defined(VMS)
+#   define fork vfork
 #endif
-    gvw->ghostview.interpreter_pid = vfork();
+    gvw->ghostview.interpreter_pid = fork();
 
     if (gvw->ghostview.interpreter_pid == 0) { /* child */
 	close(std_out[0]);
@@ -1614,6 +1449,7 @@ StartInterpreter(w)
     if (device)    GV_XtFree(device);
     gvw->ghostview.background_cleared=0;
 
+    ENDMESSAGE(StartInterpreter)
 }
 
 /* Stop the interperter, if present, and remove any Input sources. */
@@ -1623,10 +1459,12 @@ StopInterpreter(w)
     Widget w;
 {
     GhostviewWidget gvw = (GhostviewWidget) w;
+    BEGINMESSAGE(StopInterpreter)
     if (gvw->ghostview.interpreter_pid >= 0) {
+        INFMESSAGE(killing process)
 	kill(gvw->ghostview.interpreter_pid, SIGTERM);
-	wait(0);
 	gvw->ghostview.interpreter_pid = -1;
+	wait(0);
     }
     if (gvw->ghostview.interpreter_input >= 0) {
 	close(gvw->ghostview.interpreter_input);
@@ -1654,6 +1492,7 @@ StopInterpreter(w)
     }
     gvw->ghostview.busy = False;
     ChangeCursor(gvw,CURSOR_RESET);
+    ENDMESSAGE(StopInterpreter)
 }
 
 #endif /* VMS */
@@ -1682,143 +1521,247 @@ InterpreterFailed(w)
 /*###################################################################################*/
 /*###################################################################################*/
 
+void
+GhostviewDrawRectangle(w,x,y,width,height)
+  Widget w;
+  int x,y,width,height;
+{
+  GhostviewWidget gvw = (GhostviewWidget) w;
+
+  BEGINMESSAGE(GhostviewDrawRectangle)
+  XDrawRectangle(XtDisplay(w),XtWindow(w),gvw->ghostview.highlight_gc,x,y,width,height);
+  if (gvw->core.background_pixmap != XtUnspecifiedPixmap)
+     XDrawRectangle(XtDisplay(w),gvw->core.background_pixmap,gvw->ghostview.highlight_gc,x,y,width,height);
+  ENDMESSAGE(GhostviewDrawRectangle)
+}
+
+/*------------------------------------------------------------*/
+/* GhostviewOrientCoords */
+/*------------------------------------------------------------*/
+
+#define GV_O_P (1<<0)
+#define GV_O_U (1<<1)
+#define GV_O_L (1<<2)
+#define GV_O_S (1<<3)
+#define GV_O_H ((GV_O_L) | (GV_O_S))
+#define GV_O_V ((GV_O_P) | (GV_O_U))
+
+static void GhostviewOrientCoords(gvw,x,y,xP,yP,widthP,heightP,orientP)
+  GhostviewWidget gvw;
+  int x;
+  int y;
+  int *xP;
+  int *yP;
+  int *widthP;
+  int *heightP;
+  int *orientP;
+{
+  BEGINMESSAGE(GhostviewOrientCoords)
+  switch (gvw->ghostview.orientation) {
+    case XtPageOrientationPortrait:   *orientP = GV_O_P; break;
+    case XtPageOrientationUpsideDown: *orientP = GV_O_U; break;
+    case XtPageOrientationLandscape:  *orientP = GV_O_L; break;
+    case XtPageOrientationSeascape:   *orientP = GV_O_S; break;
+    default:                          *orientP = GV_O_P; break;
+  }
+  if (*orientP & GV_O_V) {
+    *widthP     = gvw->core.width-1;
+    *heightP    = gvw->core.height-1;
+    if (xP) *xP = x;
+    if (yP) *yP = y;
+  } else {
+    *widthP     = gvw->core.height-1;
+    *heightP    = gvw->core.width-1;
+    if (xP) *xP = y;
+    if (yP) *yP = x;
+  }
+  if (*widthP  < 1) *widthP=1;
+  if (*heightP < 1) *heightP=1;
+  ENDMESSAGE(GhostviewOrientCoords)
+}
+
+/*------------------------------------------------------------*/
+/* GhostviewCornersPS */
+/*------------------------------------------------------------*/
+
+static void GhostviewCornersPS(gvw,o,pslxP,psrxP,pslyP,psuyP)
+  GhostviewWidget gvw;
+  int o;
+  int *pslxP;
+  int *psrxP;
+  int *pslyP;
+  int *psuyP;
+{
+  if (o & GV_O_P) {
+    *pslxP=gvw->ghostview.llx; *psrxP=gvw->ghostview.urx;
+    *pslyP=gvw->ghostview.ury; *psuyP=gvw->ghostview.lly;
+  } else if (o & GV_O_L) {
+    *pslxP=gvw->ghostview.llx; *psrxP=gvw->ghostview.urx;
+    *pslyP=gvw->ghostview.lly; *psuyP=gvw->ghostview.ury;
+  } else if (o & GV_O_U) {
+    *pslxP=gvw->ghostview.urx; *psrxP=gvw->ghostview.llx;
+    *pslyP=gvw->ghostview.lly; *psuyP=gvw->ghostview.ury;
+  } else {
+    *pslxP=gvw->ghostview.urx; *psrxP=gvw->ghostview.llx;
+    *pslyP=gvw->ghostview.ury; *psuyP=gvw->ghostview.lly;
+  }
+}
+
+/*############################################################*/
+/* GhostviewCoordsXtoPS */
+/*############################################################*/
+
+void
+#if NeedFunctionPrototypes
+GhostviewCoordsXtoPS(
+  Widget w,
+  int wx,
+  int wy,
+  int *psxP,
+  int *psyP
+)
+#else
+GhostviewCoordsXtoPS(w,wx,wy,psxP,psyP)
+  Widget w;
+  int wx;
+  int wy;
+  int *psxP;
+  int *psyP;
+#endif
+{
+  GhostviewWidget gvw = (GhostviewWidget) w;
+  int o,x,y,pslx,psrx,psly,psuy,wrx,wly;
+
+  BEGINMESSAGE(GhostviewCoordsXtoPS)
+  GhostviewOrientCoords(gvw,wx,wy,&x,&y,&wrx,&wly,&o);
+  GhostviewCornersPS(gvw,o,&pslx,&psrx,&psly,&psuy);
+  *psxP = (pslx + ((float)x*(float)(psrx-pslx))/(float)wrx + 0.5);
+  *psyP = (psly + ((float)y*(float)(psuy-psly))/(float)wly + 0.5);
+#if 1
+  {
+    int x,y;
+    IIMESSAGE(wx,wy)
+    IIMESSAGE(*psxP,*psyP)
+    GhostviewCoordsPStoX(w,*psxP,*psyP,&x,&y);
+    IIMESSAGE(x,y)
+  }
+#endif
+  ENDMESSAGE(GhostviewCoordsXtoPS)
+}
+
+/*############################################################*/
+/* GhostviewCoordsPStoX */
+/*############################################################*/
+
+void
+#if NeedFunctionPrototypes
+GhostviewCoordsPStoX(
+  Widget w,
+  int psx,
+  int psy,
+  int *wxP,
+  int *wyP
+)
+#else
+GhostviewCoordsPStoX(w,psx,psy,wxP,wyP)
+  Widget w;
+  int psx;
+  int psy;
+  int *wxP;
+  int *wyP;
+#endif
+{
+  GhostviewWidget gvw = (GhostviewWidget) w;
+  int o,pslx,psrx,psly,psuy,wrx,wly,*xP,*yP;
+
+  BEGINMESSAGE(GhostviewCoordsPStoX)
+  GhostviewOrientCoords(gvw,0,0,NULL,NULL,&wrx,&wly,&o);
+  GhostviewCornersPS(gvw,o,&pslx,&psrx,&psly,&psuy);
+  if (o & GV_O_V) { xP = wxP; yP=wyP; }
+  else            { xP = wyP; yP=wxP; }
+  *xP = (((float)(psx-pslx)*(float)wrx)/(float)(psrx-pslx)+0.5);
+  *yP = (((float)(psy-psly)*(float)wly)/(float)(psuy-psly)+0.5);
+  IIMESSAGE(*xP,*yP)
+  ENDMESSAGE(GhostviewCoordsPStoX)
+}
+
+/*############################################################*/
+/* GhostviewGetBBOfArea */
+/*############################################################*/
+
 void
 #if NeedFunctionPrototypes
 GhostviewGetBBofArea (
-   Widget   w,
-   Position x1,
-   Position y1,
-   Position x2,
-   Position y2,
-   GhostviewReturnStruct* p
+  Widget   w,
+  int x1,
+  int y1,
+  int x2,
+  int y2,
+  GhostviewReturnStruct* p
 )
 #else
 GhostviewGetBBofArea (w,x1,y1,x2,y2,p)
-   Widget w;
-   Position x1;
-   Position y1;
-   Position x2;
-   Position y2;
-   GhostviewReturnStruct* p;
+  Widget w;
+  int x1;
+  int y1;
+  int x2;
+  int y2;
+  GhostviewReturnStruct* p;
 #endif
 {
-   GhostviewWidget gvw = (GhostviewWidget) w;
-   Position locx,locy;
+  GhostviewWidget gvw = (GhostviewWidget) w;
+  int psx1,psy1,psx2,psy2;
 
-   BEGINMESSAGE(GhostviewGetBBofArea) 
-
-   locx = (x1+x2)/2;
-   locy = (y1+y2)/2; 
-
-   p->xdpi = gvw->ghostview.xdpi;
-   p->ydpi = gvw->ghostview.ydpi;
-
-   switch (gvw->ghostview.orientation) {
-      case XtPageOrientationPortrait:
-         p->psx    = gvw->ghostview.llx + (locx * 72.0)/gvw->ghostview.xdpi;
-	 p->psy    = gvw->ghostview.ury - (locy * 72.0)/gvw->ghostview.ydpi;
-         p->width  = ((abs(x2-x1)* 72.0)/gvw->ghostview.xdpi);
-         p->height = ((abs(y2-y1)* 72.0)/gvw->ghostview.ydpi);
-   	 break;
-      case XtPageOrientationLandscape:
-	 p->psx    = gvw->ghostview.llx + (locy * 72.0)/gvw->ghostview.ydpi;
-	 p->psy    = gvw->ghostview.lly + (locx * 72.0)/gvw->ghostview.xdpi;
-         p->width  = ((abs(y2-y1)* 72.0)/gvw->ghostview.ydpi);
-         p->height = ((abs(x2-x1)* 72.0)/gvw->ghostview.xdpi);
-	 break;
-      case XtPageOrientationUpsideDown:
-	 p->psx    = gvw->ghostview.urx - (locx * 72.0)/gvw->ghostview.xdpi;
-	 p->psy    = gvw->ghostview.lly + (locy * 72.0)/gvw->ghostview.ydpi;
-         p->width  = ((abs(x2-x1)* 72.0)/gvw->ghostview.xdpi);
-         p->height = ((abs(y2-y1)* 72.0)/gvw->ghostview.ydpi);
-	 break;
-      case XtPageOrientationSeascape:
-	 p->psx    = gvw->ghostview.urx - (locy * 72.0)/gvw->ghostview.ydpi;
-	 p->psy    = gvw->ghostview.ury - (locx * 72.0)/gvw->ghostview.xdpi;
-         p->width  = ((abs(y2-y1)* 72.0)/gvw->ghostview.ydpi);
-         p->height = ((abs(x2-x1)* 72.0)/gvw->ghostview.xdpi);
-	 break;
-      case XtPageOrientationUnspecified:
-	/* never reached */
-         break;
-   }
-
-   IIMESSAGE(p->psx,p->psy)
-   IIMESSAGE(p->width,p->height)
-   FMESSAGE(p->xdpi)FMESSAGE(p->ydpi)
-
-   ENDMESSAGE(GhostviewGetBBofArea) 
+  BEGINMESSAGE(GhostviewGetBBOfArea) 
+  GhostviewCoordsXtoPS(w,x1,y1,&psx1,&psy1);
+  GhostviewCoordsXtoPS(w,x2,y2,&psx2,&psy2);
+  p->psx    = psx1 < psx2 ? psx1 : psx2;
+  p->psy    = psy1 < psy2 ? psy1 : psy2;
+  p->width  = abs(psx1-psx2)+1;
+  p->height = abs(psy1-psy2)+1;
+  p->xdpi   = gvw->ghostview.xdpi;
+  p->ydpi   = gvw->ghostview.ydpi;
+  IIMESSAGE(p->psx,p->psy)
+  IIMESSAGE(p->width,p->height)
+  ENDMESSAGE(GhostviewGetBBOfArea) 
 }
 
-/*###################################################################################*/
-/* GhostviewGetAreaofArea */
-/*###################################################################################*/
+/*############################################################*/
+/* GhostviewGetAreaOfBB */
+/*############################################################*/
 
 void
 #if NeedFunctionPrototypes
 GhostviewGetAreaOfBB (
-   Widget   w,
-   Position psx1,
-   Position psy1,
-   Position psx2,
-   Position psy2,
-   GhostviewReturnStruct* p
+  Widget   w,
+  int psx1,
+  int psy1,
+  int psx2,
+  int psy2,
+  GhostviewReturnStruct* p
 ) 
 #else
 GhostviewGetAreaOfBB (w,psx1,psy1,psx2,psy2,p)
-   Widget w;
-   Position psx1;
-   Position psy1;
-   Position psx2;
-   Position psy2;
-   GhostviewReturnStruct* p;
+  Widget w;
+  int psx1;
+  int psy1;
+  int psx2;
+  int psy2;
+  GhostviewReturnStruct* p;
 #endif
 {
-   GhostviewWidget gvw = (GhostviewWidget) w;
-   Position x1=0;
-   Position y1=0;
-   Position x2=0;
-   Position y2=0;
+  int x1,y1,x2,y2;
 
-   BEGINMESSAGE(GhostviewGetAreaOfBB) 
-
-   switch (gvw->ghostview.orientation) {
-      case XtPageOrientationPortrait:
-         x1 = ((  psx1 - gvw->ghostview.llx)*gvw->ghostview.xdpi/72.0);
-         y1 = (( -psy1 + gvw->ghostview.ury)*gvw->ghostview.ydpi/72.0);
-         x2 = ((  psx2 - gvw->ghostview.llx)*gvw->ghostview.xdpi/72.0);
-         y2 = (( -psy2 + gvw->ghostview.ury)*gvw->ghostview.ydpi/72.0);
-   	 break;
-      case XtPageOrientationLandscape:
-	 y1 = ((  psx1 - gvw->ghostview.llx)*gvw->ghostview.ydpi/72.0);
-	 x1 = ((  psy1 - gvw->ghostview.lly)*gvw->ghostview.xdpi/72.0);
-	 y2 = ((  psx2 - gvw->ghostview.llx)*gvw->ghostview.ydpi/72.0);
-	 x2 = ((  psy2 - gvw->ghostview.lly)*gvw->ghostview.xdpi/72.0);
-	 break;
-      case XtPageOrientationUpsideDown:
-	 x1 = (( -psx1 + gvw->ghostview.urx)*gvw->ghostview.xdpi/72.0);
-	 y1 = ((  psy1 - gvw->ghostview.lly)*gvw->ghostview.ydpi/72.0);
-	 x2 = (( -psx2 + gvw->ghostview.urx)*gvw->ghostview.xdpi/72.0);
-	 y2 = ((  psy2 - gvw->ghostview.lly)*gvw->ghostview.ydpi/72.0);
-	 break;
-      case XtPageOrientationSeascape:
-	 y1 = (( -psx1 + gvw->ghostview.urx)*gvw->ghostview.ydpi/72.0);
-	 x1 = (( -psy1 + gvw->ghostview.ury)*gvw->ghostview.xdpi/72.0);
-	 y2 = (( -psx2 + gvw->ghostview.urx)*gvw->ghostview.ydpi/72.0);
-	 x2 = (( -psy2 + gvw->ghostview.ury)*gvw->ghostview.xdpi/72.0);
-	 break;
-      case XtPageOrientationUnspecified:
-	/* never reached */
-         break;
-   }
-   p->psx    = x1 < x2 ? x1 : x2;
-   p->psy    = y1 < y2 ? y1 : y2;
-   p->width  = abs(x1-x2);
-   p->height = abs(y1-y2);
-   IIMESSAGE(p->psx,p->psy)
-   IIMESSAGE(p->width,p->height)
-
-   ENDMESSAGE(GhostviewGetAreaOfBB) 
+  BEGINMESSAGE(GhostviewGetAreaOfBB) 
+  GhostviewCoordsPStoX(w,psx1,psy1,&x1,&y1);
+  GhostviewCoordsPStoX(w,psx2,psy2,&x2,&y2);
+  p->psx    = x1 < x2 ? x1 : x2;
+  p->psy    = y1 < y2 ? y1 : y2;
+  p->width  = abs(x1-x2)+1;
+  p->height = abs(y1-y2)+1;
+  IIMESSAGE(p->psx,p->psy)
+  IIMESSAGE(p->width,p->height)
+  ENDMESSAGE(GhostviewGetAreaOfBB) 
 }
 
 
@@ -2081,7 +2024,7 @@ GhostviewNextPage(w)
 }
 
 /*###################################################################################*/
-/* PageOrienation Conversion Routine.
+/* Palette Conversion Routine.
  * Returns True if Conversion is successful.
  */
 /*###################################################################################*/
@@ -2104,61 +2047,6 @@ GhostviewNextPage(w)
 	    return True;					\
 	}
 
-#ifndef GV_CODE
-
-Boolean
-XmuCvtStringToPageOrientation(dpy, args, num_args, fromVal, toVal, data)
-    Display	*dpy;
-    XrmValue	*args;		/* unused */
-    Cardinal	*num_args;	/* unused */
-    XrmValue	*fromVal;
-    XrmValue	*toVal;
-    XtPointer	*data;		/* unused */
-{
-    static XrmQuark		XrmQEportrait;
-    static XrmQuark		XrmQElandscape;
-    static XrmQuark		XrmQEupsideDown;
-    static XrmQuark		XrmQEseascape;
-    static int			haveQuarks;
-    XrmQuark    q;
-    char	*str = (XPointer) fromVal->addr;
-    char        lowerName[1000];
-
-    if (str == NULL) return False;
-
-    if (!haveQuarks) {
-	XrmQEportrait   = XrmStringToQuark(XtEportrait);
-	XrmQElandscape  = XrmStringToQuark(XtElandscape);
-	XrmQEupsideDown = XrmStringToQuark(XtEupsideDown);
-	XrmQEseascape   = XrmStringToQuark(XtEseascape);
-	haveQuarks = 1;
-    }
-
-    XmuCopyISOLatin1Lowered(lowerName, str);
-
-    q = XrmStringToQuark(lowerName);
-
-    if (q == XrmQEportrait)
-	done(XtPageOrientation, XtPageOrientationPortrait);
-    if (q == XrmQElandscape)
-	done(XtPageOrientation, XtPageOrientationLandscape);
-    if (q == XrmQEupsideDown)
-	done(XtPageOrientation, XtPageOrientationUpsideDown);
-    if (q == XrmQEseascape)
-	done(XtPageOrientation, XtPageOrientationSeascape);
-
-    XtDisplayStringConversionWarning(dpy, str, XtRPageOrientation);
-    return False;
-}
-
-#endif /* GV_CODE */
-
-/*###################################################################################*/
-/* Palette Conversion Routine.
- * Returns True if Conversion is successful.
- */
-/*###################################################################################*/
-
 Boolean
 XmuCvtStringToPalette(dpy, args, num_args, fromVal, toVal, data)
     Display	*dpy;
@@ -2174,7 +2062,7 @@ XmuCvtStringToPalette(dpy, args, num_args, fromVal, toVal, data)
     static int			haveQuarks;
     XrmQuark    q;
     char	*str = (XPointer) fromVal->addr;
-    char        lowerName[1000];
+    char        lowerName[100];
 
     if (str == NULL) return False;
 
