@@ -2,7 +2,7 @@
 **
 ** Scrollbar.c
 **
-** Copyright (C) 1994-1997 Johannes Plass
+** Copyright (C) 1994, 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,29 +25,23 @@
 **
 */
 
-#ifdef VMS
-#   include <X11_DIRECTORY/IntrinsicP.h>
-#   include <X11_DIRECTORY/StringDefs.h>
-#   include <XAW_DIRECTORY/XawInit.h>
-#   include "ScrollbarP.h"
-#   include <XMU_DIRECTORY/Drawing.h>
-#   include <stdio.h> /*###jp### for printf 24.1.95 */
-#   include <stdlib.h> /*###jp### inserted 21.10.94*/
-#else
-#   include <X11/IntrinsicP.h>
-#   include <X11/StringDefs.h>
-#   include <X11/Xaw3d/XawInit.h>
-#   include "ScrollbarP.h"
-#   include <X11/Xmu/Drawing.h>
-#endif
-
-#include <math.h>
-
 /*
 #define MESSAGES
 */
 #include "message.h"
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <math.h>
+
+#include "paths.h"
+#include INC_X11(IntrinsicP.h)
+#include INC_X11(StringDefs.h)
+#include INC_XAW(XawInit.h)
+#include INC_XMU(Drawing.h)
+#include "ScrollbarP.h"
+
+
 /* Private definitions. */
 
 static char defaultTranslations[] =
