@@ -2,7 +2,7 @@
 **
 ** Scrollbar.h
 **
-** Copyright (C) 1994-19976 Johannes Plass
+** Copyright (C) 1994, 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,20 +28,16 @@
 #ifndef _Scrollbar_h
 #define _Scrollbar_h
 
+#include "paths.h"
+#include INC_XMU(Converters.h)
+#include INC_X11(Xfuncproto.h)
+
 /****************************************************************
  *
  * Scrollbar Widget
  *
  ****************************************************************/
 
-#ifdef VMS
-#   include <XMU_DIRECTORY/Converters.h>
-#   include <X11_DIRECTORY/Xfuncproto.h>
-#else
-#   include <X11/Xmu/Converters.h>
-#   include <X11/Xfuncproto.h>
-#endif
-
 /* Scrollbar resources:
 
  Name		     Class		RepType		Default Value
