@@ -2,7 +2,7 @@
 **
 ** ScrollbarP.h
 **
-** Copyright (C) 1994-1997 Johannes Plass
+** Copyright (C) 1994, 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,13 +28,9 @@
 #ifndef _ScrollbarP_h
 #define _ScrollbarP_h
 
-#ifdef VMS
-#   include <XAW_DIRECTORY/Scrollbar.h>
-#   include <XAW_DIRECTORY/ThreeDP.h>
-#else
-#   include "Scrollbar.h"
-#   include <X11/Xaw3d/ThreeDP.h>
-#endif
+#include "paths.h"
+#include INC_XAW(ThreeDP.h)
+#include "Scrollbar.h"
 
 typedef struct {
      /* public */
