@@ -2,7 +2,7 @@
 **
 ** config.h
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -58,51 +58,41 @@ correct files also. (yes Jim, this option is for you)
 #   define INTERN_RESOURCES_H       "gv_intern_res_unix.h"
 #endif
 
-/*
-----------------------------------------------------------
-Whether to show the compile date as part of the window
-title.
-----------------------------------------------------------
-*/
-
-/*
-#define SHOW_COMPILE_DATE
-*/
-
-/*
-----------------------------------------------------------
-Whether a "swap landscape" button should be provided.
-----------------------------------------------------------
-*/
-
-#define USE_SWAP_LANDSCAPE
-
 /*
 ----------------------------------------------------------
 Inline copies of optional external files.
 ----------------------------------------------------------
 */
-  
-#ifdef USE_FALLBACK_RESOURCES
-#   define FALLBACK_RESOURCES       "app-defaults.h"
+
+#ifdef USE_FALLBACK_STYLES
+#   define FALLBACK_STYLE_1_DAT         "gv_spartan.dat"
+#   define FALLBACK_STYLE_1_H           "gv_spartan.h"
 #endif
 
-#ifdef USE_FALLBACK_FILES
-#   define FALLBACK_MESSAGES        "gv_messages.h"
-#   define FALLBACK_ICON_PIXMAP     "gv_icon.xbm"
-#   define FALLBACK_SELECTED_BITMAP "gv_selected.xbm"
-#   define FALLBACK_DOCUMENT_BITMAP "gv_doc.xbm"
-#   define FALLBACK_ICON_NAME       GV
-#   define FALLBACK_SELECTED_NAME   DOT
-#   define FALLBACK_DOCUMENT_NAME   DOC
-#endif
+#define FALLBACK_ICON_PIXMAP         "gv_icon.xbm"
+#define FALLBACK_SELECTED_BITMAP     "gv_selected.xbm"
+#define FALLBACK_DOCUMENT_BITMAP     "gv_doc.xbm"
+#define FALLBACK_MARK_BITMAPS        yes_please
+#define FALLBACK_MARK_ODD_BITMAP     "gv_odd.xbm"
+#define FALLBACK_MARK_EVEN_BITMAP    "gv_even.xbm"
+#define FALLBACK_MARK_CURRENT_BITMAP "gv_current.xbm"
+#define FALLBACK_MARK_UNMARK_BITMAP  "gv_unmark.xbm"
+#define FALLBACK_MARK_EMPTY_BITMAP   "gv_empty.xbm"
+#define FALLBACK_ICON_NAME           GV
+#define FALLBACK_SELECTED_NAME       DOT
+#define FALLBACK_DOCUMENT_NAME       DOC
+#define FALLBACK_MARK_ODD_NAME       odd
+#define FALLBACK_MARK_EVEN_NAME      even
+#define FALLBACK_MARK_CURRENT_NAME   current
+#define FALLBACK_MARK_UNMARK_NAME    unmark
+#define FALLBACK_MARK_EMPTY_NAME     empty
 
 /*
 ----------------------------------------------------------
 Miscellaneous defaults
 ----------------------------------------------------------
 */
-#define GV_ERROR_PRINT_FAIL	"'%s' Command failed."
+#define GV_ERROR_PRINT_FAIL	"Printing via\n '%s'\nfailed."
 #define GV_PRINT_MESSAGE	"Print Command:"
 #define GV_PRINT_MARKED_MESSAGE	"Print Marked Pages"
 #define GV_PRINT_PAGE_MESSAGE	"Print Current Page"
@@ -114,7 +104,6 @@ Miscellaneous defaults
 #define GV_SAVE_MESSAGE		"Save"
 #define GV_AUTO_RESIZE_YES	"Variable Size"
 #define GV_AUTO_RESIZE_NO	"Fixed Size"
-#define GV_LABEL_MENU_NAME	"labelMenu"
 #define GV_MINIMUM_SIZE		300
 #define MAX_LOCATOR_LENGTH	48
 #define TOC3D_INITIAL_HEIGHT	30
@@ -122,7 +111,6 @@ Miscellaneous defaults
 
 #define	GV_MAX_FILENAME_LENGTH	256
 #define GV_APPLICATION_NAME	"gv"
-#define GV_INTERN_NAME		"GVintern"
 #define GV_CLASS		"GV"
 
 #ifdef VMS
