@@ -2,7 +2,7 @@
 **
 ** d_xtmem.c
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -33,6 +33,8 @@
 #include "paths.h"
 #include INC_X11(Intrinsic.h) 
 
+#include "d_xtmem.h"
+
 #define GV_MEMDEBUG_OUT stderr
 
 typedef struct _mem {
