@@ -2,7 +2,7 @@
 **
 ** dialog.c
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -39,15 +39,16 @@
 #include INC_X11(StringDefs.h)
 #include INC_X11(Xos.h)
 #include INC_XAW(AsciiText.h)
-#include INC_XAW(Command.h)
 #include INC_XAW(Label.h)
 #include INC_X11(Shell.h)
 #include "Aaa.h"
+#include "Button.h"
 #include "Ghostview.h"
 
+#include "types.h"
 #include "main_resources.h"
 #include "main_globals.h"
-#include "popup_misc.h"
+#include "popup.h"
 #include "widgets_misc.h"
 
 #ifndef max
@@ -60,7 +61,6 @@
 #define DIALOG_POPUP_NAME	"dialogPopup"
 #define DIALOG_POPUP		dialogpopup
 
-#include "gv.h"
 #include "note.h"
 #include "dialog.h"
 
@@ -99,7 +99,7 @@ void cb_popupDialogPopup(w, client_data, call_data)
       if (!dialogPopupCreated) makeDialogPopup();
       preferedButton=DIALOG_BUTTON_CANCEL;
       action_preferDialogPopupButton(NULL, NULL, NULL, NULL);
-      popup_positionPopup(DIALOG_POPUP,fileButton,POPUP_POSITION_POS,85,25);
+      popup_positionPopup(dialogpopup,viewFrame,POPUP_POSITION_POS,4,4);
       XtPopup(DIALOG_POPUP,XtGrabNone);
       dialogPopupVisible=True;
    }
@@ -141,7 +141,10 @@ action_preferDialogPopupButton(w, event, params, num_params)
 
 static char *tabTrans =
 "\
-<Key>Tab: GV_TogDialPrefBut()\
+<Key>Tab: GV_TogDialPrefBut()\n\
+<EnterNotify>:no-op()\n\
+<LeaveNotify>:no-op()\n\
+<MapNotify>:display-caret(on,always)\
 ";
 
 void makeDialogPopup()
@@ -161,7 +164,6 @@ void makeDialogPopup()
 					n=0;
         XtSetArg(args[n], XtNresizeWidth, True);	n++;
         XtSetArg(args[n], XtNresizeHeight, True);	n++;
-        XtSetArg(args[n], XtNconditionedResize, False);	n++;
         XtSetArg(args[n], XtNmaximumWidth, 0);	 	n++;
         XtSetArg(args[n], XtNmaximumHeight,0);	 	n++;
    dialogAaa = XtCreateManagedWidget("dialogAaa",aaaWidgetClass,DIALOG_POPUP,args,n);
@@ -170,10 +172,10 @@ void makeDialogPopup()
    XtCreateManagedWidget("message", labelWidgetClass,dialogAaa, args, n);
 
         				n=0;
-   doneButton=XtCreateManagedWidget("done",commandWidgetClass,dialogAaa,args,n);
-   cancelButton=XtCreateManagedWidget("cancel",commandWidgetClass,dialogAaa,args,n);
+   doneButton=XtCreateManagedWidget("done",buttonWidgetClass,dialogAaa,args,n);
+   cancelButton=XtCreateManagedWidget("cancel",buttonWidgetClass,dialogAaa,args,n);
  
-   dialogText=widgets_createLabeledTextField("dialog_text", dialogAaa);
+   dialogText=widgets_createLabeledLineTextField("dialog_text", dialogAaa);
 
    XtRealizeWidget(DIALOG_POPUP);
 
@@ -188,12 +190,12 @@ void makeDialogPopup()
    XtSetValues(DIALOG_POPUP, args, n);
 
    /*>>>*/
+      if (!tab_trans) tab_trans=XtParseTranslationTable(tabTrans);
+      XtOverrideTranslations(dialogText,tab_trans);
       XtSetKeyboardFocus(dialogAaa,dialogText);
 					      n=0;
       XtSetArg(args[n],XtNdisplayCaret,True); n++;
       XtSetValues(dialogText, args, n);
-      if (!tab_trans) tab_trans=XtParseTranslationTable(tabTrans);
-      XtOverrideTranslations(dialogText,tab_trans);
    /*<<<*/
 
 
@@ -284,20 +286,20 @@ void DialogPopupSetText(s)
     XtSetArg(args[n], XtNstring, &os);		n++;
     XtSetArg(args[n], XtNwidth, &width);	n++;
     XtGetValues(response, args, n);
-    if (!os) os="";
-    if (strcmp(os,s)) {
     						n=0;
-       nw= ((font->max_bounds.width+font->min_bounds.width)*((Dimension)strlen(s))+1)/2
-	   +lm+rm+DIALOG_POPUP_FREE_SPACE+2*bw;
-       if (nw<DIALOG_POPUP_TEXT_MIN_WIDTH) nw=DIALOG_POPUP_TEXT_MIN_WIDTH;
-       if (nw>width) {
-	  INFIMESSAGE(new width,nw)
-	  XtSetArg(args[n], XtNwidth,nw);	n++;
-       }
-       XtSetArg(args[n], XtNstring, s);		n++;
-       XtSetValues(response, args, n);
- 
+    nw= ((font->max_bounds.width+font->min_bounds.width)*((Dimension)strlen(s))+1)/2
+      +lm+rm+DIALOG_POPUP_FREE_SPACE+2*bw;
+    if (nw<DIALOG_POPUP_TEXT_MIN_WIDTH) nw=DIALOG_POPUP_TEXT_MIN_WIDTH;
+    if (nw>width) {
+      INFIMESSAGE(new width,nw)
+      XtSetArg(args[n], XtNwidth,nw);	n++;
     }
+    XtSetArg(args[n], XtNstring, s);		n++;
+    XtSetValues(response, args, n);
+    						n=0;
+    XtSetArg(args[n], XtNinsertPosition,strlen(s));	n++;
+    XtSetValues(response, args, n);
+ 
     ENDMESSAGE(DialogPopupSetText)
 }
 
