@@ -2,7 +2,7 @@
 **
 ** doc_misc.c
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -39,20 +39,26 @@
 #include "Ghostview.h"
 #include "main_resources.h"
 
-#include "gv.h"
+#include "types.h"
+extern Media* gv_medias;
 #include "ps.h"
+#include "doc_misc.h"
 
 /*##################################################################*/
 /* doc_scanFile */
 /*##################################################################*/
 
 int
-doc_scanFile(fPP,docP,file_name,file_name_scanP,cmd_scan_pdf)
+doc_scanFile(fPP,docP,filename,filename_raw,filename_dscP,cmd_scan_pdf,filename_uncP,cmd_uncompress,scanstyle)
    FILE ** fPP;
    Document *docP;
-   String file_name;
-   String *file_name_scanP;
+   String filename;
+   String filename_raw;
+   String *filename_dscP;
    String cmd_scan_pdf;
+   String *filename_uncP;
+   String cmd_uncompress;
+   int scanstyle;
 {
    Document d;
    int i,ret;
@@ -60,9 +66,10 @@ doc_scanFile(fPP,docP,file_name,file_name_scanP,cmd_scan_pdf)
    BEGINMESSAGE(doc_scanFile)
    d = (Document)NULL;
    ret = 0;
-   if (*fPP && file_name && app_res.respect_dsc) d = psscan(fPP,file_name,file_name_scanP,cmd_scan_pdf);
+   if (*fPP && filename) 
+     d = psscan(fPP,filename,filename_raw,filename_dscP,cmd_scan_pdf,filename_uncP,cmd_uncompress,scanstyle);
    if (d) {
-      d->labels_useful=1;
+      d->labels_useful=0;
       d->structured   =0;
       if ((!d->epsf && d->numpages>0) || (d->epsf && d->numpages>1)) {
          ret = d->structured = 1;
@@ -103,61 +110,136 @@ doc_putPageInRange(d,pagenumber)
    return pagenumber;
 }    
 
+/*############################################################*/
+/* doc_mediaIsOk */
+/*############################################################*/
+
+int 
+doc_mediaIsOk (d,pagenumber,m)
+  Document d;
+  int pagenumber;
+  int m;
+{
+  int r=1;
+  int bbox = d ? d->nummedia : 0;
+  BEGINMESSAGE(doc_mediaIsOk)
+  if (m==bbox) {
+    int l;
+    if (doc_boundingBoxOfPage(d,pagenumber,&l,&l,&l,&l) == MEDIA_ID_INVALID)
+      r = 0;
+  } else if (m==MEDIA_ID_INVALID)
+      r = 0;
+  ENDMESSAGE(doc_mediaIsOk)
+  return(r);
+}
+
+/*############################################################*/
+/* doc_boundingBoxOfPage */
+/*############################################################*/
+
+int
+doc_boundingBoxOfPage(d,pagenumber,llxP,llyP,urxP,uryP)
+  Document d;
+  int pagenumber;
+  int *llxP,*llyP,*urxP,*uryP;
+{
+  int *dbb = NULL;
+  int retry;
+
+  BEGINMESSAGE(doc_boundingBoxOfPage)
+
+  if (!d) {
+    INFMESSAGE(no document) ENDMESSAGE(doc_preferredMediaOfPage)
+    return(MEDIA_ID_INVALID);
+  }
+
+  retry=1;
+# define BB_VALID (dbb[URX]>dbb[LLX] && dbb[URY]>dbb[LLY])
+  if (d->structured && 0<=pagenumber && pagenumber<(int)d->numpages)
+    { dbb = d->pages[pagenumber].boundingbox;  if BB_VALID retry=0; }
+  if (retry && d->structured)
+    { dbb = d->default_page_boundingbox; if BB_VALID retry=0; }
+  if (retry)
+    { dbb = d->boundingbox;              if BB_VALID retry=0; }
+# undef BB_VALID
+  if (!retry) {
+    *llxP=dbb[LLX]; *llyP=dbb[LLY]; *urxP=dbb[URX]; *uryP=dbb[URY];
+    INFMESSAGE(bounding box)
+    ENDMESSAGE(doc_boundingBoxOfPage)
+    return(d->nummedia);
+  }
+  INFMESSAGE(MEDIA_ID_INVALID)
+  ENDMESSAGE(doc_boundingBoxOfPage)
+  return(MEDIA_ID_INVALID);
+}
+
 /*############################################################*/
 /* doc_preferredMediaOfPage */
 /*############################################################*/
 
 int
 doc_preferredMediaOfPage(d,pagenumber,llxP,llyP,urxP,uryP)
-   Document d;
-   int pagenumber;
-   int *llxP,*llyP,*urxP,*uryP;
+  Document d;
+  int pagenumber;
+  int *llxP,*llyP,*urxP,*uryP;
 {
-   int *dbb = NULL;
-   struct documentmedia *media = NULL;
-   int retry;
+# define BB_VALID (dbb[URX]>dbb[LLX] && dbb[URY]>dbb[LLY])
+  int *dbb = NULL;
+  Media media = NULL,dm;
+  int m=MEDIA_ID_INVALID;
+  int found,i,j;
 
-   BEGINMESSAGE(doc_preferredMediaOfPage)
+  BEGINMESSAGE(doc_preferredMediaOfPage)
 
-   if (!d) {
-      INFMESSAGE(no document) ENDMESSAGE(doc_preferredMediaOfPage)
-      return(MEDIA_ID_INVALID);
-   }
+  if (!d) {
+    INFMESSAGE(no document) ENDMESSAGE(doc_preferredMediaOfPage)
+    return(MEDIA_ID_INVALID);
+  }
 
-   retry=1;
-#  define BB_VALID (dbb[URX]>dbb[LLX] && dbb[URY]>dbb[LLY])
-   if (d->structured && 0<=pagenumber && pagenumber<(int)d->numpages)
-      { dbb = d->pages[pagenumber].boundingbox;  if BB_VALID retry=0; }
-   if (retry && d->structured)
-      { dbb = d->default_page_boundingbox; if BB_VALID retry=0; }
-   if (retry && d->epsf)
-      { dbb = d->boundingbox;              if BB_VALID retry=0; }
-#  undef BB_VALID
-   if (!retry) {
-      INFMESSAGE(MEDIA_ID_BB) ENDMESSAGE(doc_preferredMediaOfPage)
-      *llxP=dbb[LLX]; *llyP=dbb[LLY]; *urxP=dbb[URX]; *uryP=dbb[URY];
-      return(MEDIA_ID_BB);
-   }
+  found=0;
+  if (!found && d->structured && 0<=pagenumber && pagenumber<(int)d->numpages)
+    { media = (Media) d->pages[pagenumber].media;  if (media) found=1; }
+  if (!found)
+    { media = (Media) d->default_page_media; if (media) found=1; }
+  if (d->structured && 0<=pagenumber && pagenumber<(int)d->numpages)
+    { dbb = d->pages[pagenumber].boundingbox;  if BB_VALID found=-1; }
+  if (!found && d->structured)
+    { dbb = d->default_page_boundingbox; if BB_VALID found=-1; }
+  if (!found)
+    { dbb = d->boundingbox;              if BB_VALID found=-1; }
+# undef BB_VALID
 
-   if (d->structured && 0<=pagenumber && pagenumber<(int)d->numpages)
-      { media = d->pages[pagenumber].media;  if (media) retry=0; }
-   if (retry)
-      { media = d->default_page_media; if (media) retry=0; }
-   if (!retry) {
-      struct documentmedia *tmp;
-      int i,m=MEDIA_ID_INVALID;
-      for (tmp=d->media,i=0; i<d->nummedia; i++,tmp++)
-          if (media==tmp) { m = i; break; }
-      if (m!=MEDIA_ID_INVALID) {
-         INFIMESSAGE(doc prefers media,m) ENDMESSAGE(doc_preferredMediaOfPage)
-         *llxP=*llyP=0; *urxP=media->width-1; *uryP=media->height-1;
-         return(m);
-      }
-   }
-   INFMESSAGE(MEDIA_ID_INVALID) ENDMESSAGE(doc_preferredMediaOfPage)
-   return(MEDIA_ID_INVALID);
-} 
+  /* try to map bounding box to standard or document pagemedia */
+  if (found==-1 && dbb[LLX]==0 && dbb[LLY]==0) {
+    for (dm=(Media) d->media,i=0; !media && i<d->nummedia; i++,dm++)
+      if ((dm->width==dbb[URX] && dm->height==dbb[URY]) ||
+	  (dm->width+1==dbb[URX] && dm->height+1==dbb[URY])) media=dm;
+    for (j=0; gv_medias[j] && !media ; j++) {
+       dm = gv_medias[j];
+       if ((dm->width==dbb[URX] && dm->height==dbb[URY]) ||
+	   (dm->width+1==dbb[URX] && dm->height+1==dbb[URY])) media=dm;
+    }
+    if (media) found=1;
+  }
 
+  if (found==-1) {
+    *llxP=dbb[LLX]; *llyP=dbb[LLY]; *urxP=dbb[URX]; *uryP=dbb[URY];
+    INFMESSAGE(bounding box)
+    m=d->nummedia;
+  } else if (found==1) {
+    for (dm=(Media)d->media,i=0; i<d->nummedia; i++,dm++)
+      if (media==dm) { m = i; break; }
+    if (m==MEDIA_ID_INVALID) for (i=0; gv_medias[i] ; i++)
+      if (media==gv_medias[i]) { m = i+d->nummedia; break; }
+    if (m!=MEDIA_ID_INVALID) {
+      INFIMESSAGE(doc prefers media,m)
+      *llxP=*llyP=0; *urxP=media->width-1; *uryP=media->height-1;
+    }
+  }
+  INFIMESSAGE(returning media number,m)
+  ENDMESSAGE(doc_preferredMediaOfPage)
+  return(m);
+}
 
 /*##################################################################*/
 /* doc_preferredOrientationOfPage */
@@ -171,13 +253,13 @@ doc_preferredOrientationOfPage(d,page)
    int o;
 
    BEGINMESSAGE(doc_preferredOrientationOfPage)
-   o  = O_NONE;
+   o  = O_UNSPECIFIED;
    if (d) {
-      if (d->structured && 0<=page && page<(int)d->numpages && d->pages[page].orientation != O_NONE)
+      if (d->structured && 0<=page && page<(int)d->numpages && d->pages[page].orientation != O_UNSPECIFIED)
          o = d->pages[page].orientation;
-      else if (d->default_page_orientation != O_NONE)
+      else if (d->default_page_orientation != O_UNSPECIFIED)
          o = d->default_page_orientation;
-      else if (d->orientation != O_NONE)
+      else if (d->orientation != O_UNSPECIFIED)
          o = d->orientation;
    }
    ENDMESSAGE(doc_preferredOrientationOfPage)
@@ -199,7 +281,6 @@ doc_convStringToPage(d,pageLabel)
    BEGINMESSAGE(doc_convStringToPage)
    page=-1;
    if (pageLabel && d && d->labels_useful) for (i=0; i<d->numpages; i++) {
-      INFMESSAGE(0)
       if (d->pageorder == DESCEND) j = (int)d->numpages-1-i;
       else                         j = i;
       if (!strcmp(pageLabel,d->pages[j].label)) { page=i; break; }
@@ -242,7 +323,7 @@ doc_convDocOrientToXtOrient(orientation,swapLandscape)
      if (swapLandscape) {INFMESSAGE(landscape)ret = XtPageOrientationLandscape;}
       else              {INFMESSAGE(seascape)ret = XtPageOrientationSeascape;}
       break;
-   case O_NONE:
+   case O_UNSPECIFIED:
       INFMESSAGE(unspecified)
       ret = XtPageOrientationUnspecified;
       break;
@@ -305,8 +386,8 @@ doc_convStringToPageMedia(d,mstr)
    if (media==MEDIA_ID_INVALID) {
       int num_doc_papersizes=0;
       if (d) num_doc_papersizes = d->nummedia;
-      for (i=0; papersizes[i].name; i++)
-	  if (!XmuCompareISOLatin1(mstr,papersizes[i].name)) media = num_doc_papersizes+i;
+      for (i=0; gv_medias[i]; i++)
+	  if (gv_medias[i]->used && !XmuCompareISOLatin1(mstr,gv_medias[i]->name)) media = num_doc_papersizes+i;
    }
    IMESSAGE(media)
    ENDMESSAGE(doc_convStringToPageMedia)
