@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -32,9 +32,13 @@ extern int			doc_scanFile (
 #if NeedFunctionPrototypes
    FILE **,
    Document*,
-   String, 
+   String,
+   String,
    String*,
-   String
+   String,
+   String*,
+   String,
+   int     /* scanstyle */
 #endif
 );
 
@@ -45,6 +49,25 @@ extern int			doc_putPageInRange (
 #endif
 );
 
+extern int			doc_mediaIsOk (
+#if NeedFunctionPrototypes
+    Document,
+    int,
+    int
+#endif
+);
+
+extern int			doc_boundingBoxOfPage (
+#if NeedFunctionPrototypes
+    Document,
+    int,
+    int*,
+    int*,
+    int*,
+    int*
+#endif
+);
+
 extern int			doc_preferredMediaOfPage (
 #if NeedFunctionPrototypes
     Document,
