@@ -2,7 +2,7 @@
 **
 ** error.h
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -39,10 +39,12 @@
 #      include <perror.h>
 #   endif
 #else
+#if 0 /* unused ###jp###, 02.06.97 */
 #   ifndef CSRG_BASED /* for __FreeBSD__ */
        extern int sys_nerr;
        extern char *sys_errlist[];
 #   endif
+#endif 
 #   include <errno.h>
     /* BSD 4.3 errno.h does not declare errno */
     extern int errno;
