@@ -2,7 +2,7 @@
 **
 ** file.c
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -34,6 +34,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
+#include <ctype.h>
 
 #ifdef VMS
 #   include <stat.h>
@@ -45,11 +46,20 @@
 #include "paths.h"
 #include INC_X11(Intrinsic.h)
 
+#include "types.h"
+#include "config.h"
 #include "d_memdebug.h"
 #include "file.h"
 #include "main_resources.h"
 #include "main_globals.h"
 
+#ifdef VMS
+#   include "strcasecmp.h"
+#else
+#   include <string.h>
+#endif
+
+
 /*############################################################*/
 /* file_getDirOfPath */
 /*############################################################*/
@@ -171,17 +181,21 @@ file_getTmpFilename(baseDirectory,baseFilename)
    if (pos) { *pos='\0'; tmpExt = pos+1; }
    else tmpExt = "";
 
-   if (strlen(tmpName)>20) *(tmpName+20) = '\0';
-   if (strlen(tmpExt) >8)  *(tmpExt+8)   = '\0';
+/* Limit filename to 39 characters (excluding dir and .tmp).
+   This is required for VMS, but is also reasonable for Unix. */
+   if (strlen(tmpName)+strlen(tmpExt)>23) {
+      if (strlen(tmpExt)>11) tmpExt[11] = '\0';     /* allow .ps_page_nnn */
+      if (strlen(tmpName)+strlen(tmpExt)>23) tmpName[23-strlen(tmpExt)] = '\0';
+   }
    {
       struct stat s;
       int no_such_file;
       int i=1;
       do {
 #ifdef VMS
-         sprintf(tempFilename,"%sgv_%s_%s_%ld_%d.tmp",tmpDirBuf,tmpName,tmpExt,time(NULL),i);
+         sprintf(tempFilename,"%sgv_%lx_%x_%s_%s.tmp",tmpDirBuf,time(NULL),i,tmpName,tmpExt);
 #else
-         sprintf(tempFilename,"%sgv_%s.%s_%ld_%d.tmp",tmpDirBuf,tmpName,tmpExt,time(NULL),i);
+         sprintf(tempFilename,"%sgv_%lx_%x_%s.%s.tmp",tmpDirBuf,time(NULL),i,tmpName,tmpExt);
 #endif
          file_translateTildeInPath(tempFilename);
          no_such_file = stat(tempFilename,&s);
@@ -223,25 +237,138 @@ file_translateTildeInPath(path)
    ENDMESSAGE(file_translateTildeInPath)
 }
 
+/*############################################################*/
+/* file_fileIsDir */
+/*############################################################*/
+
+int file_fileIsDir(fn)
+  char *fn;
+{
+  struct stat s;
+  int r=0;
+  char *c;
+  BEGINMESSAGE(file_fileIsNotUseful)
+  if (fn) {
+#ifdef VMS
+     c = strrchr(fn,']');
+     if (c && (!*(c+1) || isspace(*(c+1)))) r = 1;
+     if (!r) {
+       c = strrchr(fn,':');
+       if (c && (!*(c+1) || isspace(*(c+1)))) r = 1;
+     }
+#else
+     c = strrchr(fn,'/');
+     if (c && (!*(c+1) || isspace(*(c+1)))) r = 1;
+#endif
+     if (!r && !stat(fn,&s)  && (S_ISDIR(s.st_mode))) r=1;
+  }
+  IMESSAGE(r)
+  ENDMESSAGE(file_fileIsDir)
+  return(r);
+}
+
 /*############################################################*/
 /* file_fileIsNotUseful */
 /*############################################################*/
 
 int
 file_fileIsNotUseful(fn)
-   char *fn;
+  char *fn;
 {
-   struct stat s;
-   int retval=0;
-   BEGINMESSAGE(file_fileIsNotUseful)
-   if (stat(fn,&s)  || (S_ISDIR(s.st_mode)) || s.st_size==0) {
-      retval=1;
-   }
-   IMESSAGE(retval)
-   ENDMESSAGE(file_fileIsNotUseful)
-   return(retval);
+  struct stat s;
+  int r=0;
+  BEGINMESSAGE(file_fileIsNotUseful)
+  if (!fn || stat(fn,&s)  || (S_ISDIR(s.st_mode)) || s.st_size==0) r=1;
+  IMESSAGE(r)
+  ENDMESSAGE(file_fileIsNotUseful)
+  return(r);
+}
+
+/*############################################################*/
+/* file_pdfname2psname */
+/* If the file ends in .pdf, change this to .ps.*/
+/* Return pointer to temp copy if changed, else to input string. */
+/*############################################################*/
+ 
+char *
+file_pdfname2psname(name)
+  char *name;
+{
+  char *e;
+
+  BEGINMESSAGE(file_pdfname2psname)
+  if (!name) {
+     ENDMESSAGE(file_pdfname2psname)
+     return(name);
+  }
+  INFSMESSAGE(in,name)
+# ifdef VMS
+    e = strrchr(name,';');
+    if (!e) e = name+strlen(name);
+    else *e = '\0';
+# else
+    e = name+strlen(name);
+# endif
+  if ((e-name)-4 >= 0) {
+    e -= 4;
+    if (!strcasecmp(e,".pdf")) {
+#     ifdef VMS
+        strcpy(e,".PS");
+#     else
+        strcpy(e,".ps");
+#     endif
+    }
+  }
+  INFSMESSAGE(out,name)
+  ENDMESSAGE(file_pdfname2psname)
+  return(name);
 }
 
 
+/*############################################################*/
+/* file_getUsefulName */
+/*############################################################*/
+ 
+char *
+file_getUsefulName(name)
+  char *name;
+{
+  char *e,*c;
+  String mext,ext;
 
+  BEGINMESSAGE(file_getUsefulName)
+  if (!name) {
+    ENDMESSAGE(file_getUsefulName)
+    return(name);
+  }
+  INFSMESSAGE(in,name)
+# ifdef VMS
+    e = strrchr(name,';');
+    if (e) *e = '\0';
+# endif
+  c = e = strrchr(name,'.');
+  if (!e) {
+     ENDMESSAGE(file_getUsefulName)
+     return(name);
+  }
+  mext = ext = GV_XtNewString(e);
+  while (*ext) { *ext = tolower(*ext); ext++; }
+  if      (!strncmp(mext,".gz",3))  ext = "";
+  else if (!strncmp(mext,".z",2))   ext = "";
+  else if (!strncmp(mext,".ps",3))  ext = ".ps";
+  else if (!strncmp(mext,".pdf",4)) ext = ".pdf";
+  else                              ext = e;
+  if (e == name && !(*ext)) {
+    ENDMESSAGE(file_getUsefulName)
+    return(name);
+  }
+  strcpy(mext,ext);
+  ext=mext;
+  while (*ext && *e) *ext++=*e++;
+  strcpy(c,mext);
+  GV_XtFree(mext);
+  INFSMESSAGE(out,name)
+  ENDMESSAGE(file_getUsefulName)
+  return(name);
+}
 
