@@ -2,7 +2,7 @@
 **
 ** info.c
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -37,10 +37,10 @@
 #include "paths.h"
 #include INC_X11(Intrinsic.h)
 #include INC_X11(StringDefs.h)
-#include INC_XAW(Command.h)
 #include INC_XAW(Form.h)
 #include INC_XAW(AsciiText.h)
 #include INC_X11(Shell.h)
+#include "Button.h"
 
 /*### Application dependencies ##############################################*/
 
@@ -48,7 +48,8 @@
 #define  INFO_APPLIC_CONTEXT		app_con
 #define  INFO_POPUP			infopopup
 #define  INFO_POPUP_NAME		"infoPopup"
-#include "gv.h"
+
+#include "types.h"
 #include "popup.h"
 #include "info.h"
 #include "main_resources.h"
@@ -204,7 +205,7 @@ void makeInfoPopup()
             XtSetArg(args[n], XtNbottom, XtChainBottom);			n++;
             XtSetArg(args[n], XtNleft, XtChainLeft);				n++;
             XtSetArg(args[n], XtNright, XtChainRight);				n++;
-    infodismiss = XtCreateManagedWidget("dismiss", commandWidgetClass,infoform,args,n);
+    infodismiss = XtCreateManagedWidget("dismiss", buttonWidgetClass,infoform,args,n);
             XtAddCallback(infodismiss, XtNcallback, cb_popdownInfoPopup,NULL);
    
 										n=0;
