@@ -4,7 +4,7 @@
 ** Version 2.6
 ** Mai 1996
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -42,6 +42,7 @@
 #endif
 
 #include<stdlib.h>
+#include<stdio.h>
 
 #if defined(__STDC__) || defined(__ALPHA)
 #   define _MESSAGE_USE_STDC_
