@@ -2,7 +2,7 @@
 **
 ** misc.h
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -28,6 +28,15 @@
 #ifndef	_GV_MISC_H_
 #define	_GV_MISC_H_
 
+extern void			misc_drawEyeGuide (
+#if NeedFunctionPrototypes
+  Widget,
+  int,
+  int,
+  int
+#endif
+);
+
 extern void			misc_savePagePosition (
 #if NeedFunctionPrototypes
 #endif
@@ -45,12 +54,25 @@ extern void			misc_resetPagePosition (
 #endif
 );
 
+extern void			misc_setPageMarker (
+#if NeedFunctionPrototypes
+    int, /* entry */
+    int	 /* kind */
+#endif
+);
+
 extern String			misc_testFile (
 #if NeedFunctionPrototypes
     String	/* name */
 #endif
 );
 
+extern String			misc_changeFile (
+#if NeedFunctionPrototypes
+    String	/* name */
+#endif
+);
+
 extern String			close_file (
 #if NeedFunctionPrototypes
     FILE *,	/* file */
@@ -81,6 +103,11 @@ extern void			setup_layout_ghostview (
 #endif
 ); 
 
+extern void			misc_buildPagemediaMenu (
+#if NeedFunctionPrototypes
+#endif
+); 
+
 extern int			catch_Xerror (
 #if NeedFunctionPrototypes
     Display*,
