@@ -2,7 +2,7 @@
 **
 ** popup.h
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -43,5 +43,20 @@ extern void cb_popdownPopup(
 #endif
 );
 
+extern void			popup_positionPopup (
+#if NeedFunctionPrototypes
+    Widget,
+    Widget,
+    int,
+    int,
+    int
+#endif
+);
+
+#define POPUP_POSITION_POS		1
+#define POPUP_POSITION_CENTER		2
+#define POPUP_POSITION_POS_CENTER	3
+#define POPUP_POSITION_POS_NW		4
+
 #define _GV_POPUP_H_
 #endif  /* _GV_POPUP_H_ */
