@@ -35,12 +35,6 @@
  *
 */
 
-/*
- * this code was modified in 12/93 by Johannes Plass,
- * plass@thep.physik.uni-mainz.de
- *
-*/
-
 /*
  * > Believe it or not--even Adobe doesn't know how to produce correct
  * > PS-files. Their Acrobat Reader sometimes starts including other files
@@ -70,6 +64,8 @@
 #   include "strcasecmp.h"
 #   define unlink remove
 #   define USE_FTELL_FOR_FILEPOS
+#else
+#   include <string.h>
 #endif
 
 #ifndef SEEK_SET
@@ -85,12 +81,13 @@
 #include INC_X11(Xos.h)                /* #includes the appropriate <string.h> */
 #include INC_X11(Xfuncs.h)
 
+#include "types.h"
 #include "file.h"
-#include "gv.h"
 #include "misc.h"
 #include "ps.h"
 #include "note.h"
 #include "d_memdebug.h"
+extern Media *gv_medias;
 
 #ifdef BSD4_2
 #define memset(a,b,c) bzero(a,c)
@@ -104,21 +101,23 @@
 
 /* list of standard paper sizes from Adobe's PPD. */
 
+#if 0
 struct documentmedia papersizes[] = {
+    {"BBox",	         0,  0},
     {"Letter",		 612,  792},
     {"LetterSmall",	 612,  792},
-    {"Tabloid",		 792, 1224},
-    {"Ledger",		1224,  792},
     {"Legal",		 612, 1008},
     {"Statement",	 396,  612},
+    {"Tabloid",		 792, 1224},
+    {"Ledger",		1224,  792},
     {"Executive",	 540,  720},
-    {"A0",		2384, 3371},
-    {"A1",		1685, 2384},
-    {"A2",		1192, 1685},
-    {"A3",		 843, 1192},
-    {"A4",		 596,  843},
-    {"A4Small",		 596,  843},
-    {"A5",		 421,  596},
+    {"A0",		2384, 3370},
+    {"A1",		1684, 2384},
+    {"A2",		1191, 1684},
+    {"A3",		 842, 1191},
+    {"A4",		 595,  842},
+    {"A4Small",		 595,  842},
+    {"A5",		 420,  595},
     {"B4",		 729, 1032},
     {"B5",		 516,  729},
     {"Folio",		 612,  936},
@@ -126,6 +125,8 @@ struct documentmedia papersizes[] = {
     {"10x14",		 720, 1008},
     {  NULL,		   0,    0}
 };
+#endif
+
 
 #if NeedFunctionPrototypes
 #   define PT(aaa) aaa
@@ -280,6 +281,61 @@ static char    *empty_string = "";
  *    installed a wing from a DC-2.
  */
 
+#ifndef VMS
+#  define ps_system system
+#else
+
+/*############################################################
+ * ps_system        (VMS only, otherwise same as system())
+ * Similar to system(), except it allows I/O redirection with <in >out
+ * at the end of the line, and returns a more Unix-compatible status (0=OK).
+ * The I/O redirection is very rudimentary. >> and | are NOT supported.
+ * No error checking is performed, though hopefully the command will return
+ * a reasonable error message when it can't access the file.
+ * < or > in the command (eg. in directory spec), unless followed by another
+ * < or >, will always be interpreted as I/O redirection, so be careful!
+ *############################################################*/
+
+int ps_system(cmd)
+  char *cmd;
+{
+  char *inpf=NULL,*outf=NULL,*cf=NULL,*s,*t;
+  int status;
+
+  if (!cmd) cmd="";
+  cmd = PS_XtNewString(cmd);
+  s = strrchr(cmd,'>'); if (s) { outf = s+1; *s='\n'; }
+  s = strrchr(cmd,'<'); if (s) { inpf = s+1; *s='\n'; }
+  while ((s=strrchr(cmd,'\n'))) *s='\0';
+  if (outf) {
+    for (s=t=outf; *s; s++) if (!isspace(*s)) *t++=*s; 
+    *t='\0';
+    if (!(*outf)) outf=NULL;
+  }
+  if (inpf) {
+    for (s=t=inpf; *s; s++) if (!isspace(*s)) *t++=*s;
+    *t='\0';
+    if (!(*inpf)) inpf=NULL;
+  }
+  if (outf || inpf) {
+    FILE *com;
+    cf=file_getTmpFilename (NULL, outf ? outf : inpf);
+    com=fopen(cf, "w");
+    if (!com) return 1;
+    if (inpf) fprintf(com, "$DEFINE_/NOLOG SYS$INPUT  %s\n",inpf);
+    if (outf) fprintf(com, "$DEFINE_/NOLOG SYS$OUTPUT %s\n",outf);
+    fprintf(com,"$%s\n",cmd);
+    fclose(com);
+    PS_XtFree(cmd);
+    cmd = PS_XtMalloc((1+strlen(cf)+1)*sizeof(char));
+    sprintf(cmd,"@%s",cf);
+  }
+  status=system(cmd);
+  if (cf) unlink(cf);
+  PS_XtFree(cmd);
+  return((status&7)==1 ? 0 : status);
+}
+#endif
 
 /*-----------------------------------------------------------*/
 
@@ -295,13 +351,17 @@ static void ps_dynMemExhaust()
 /*###########################################################*/
 
 struct document *
-psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
+psscan(fileP,filename,filename_raw,filename_dscP,cmd_scan_pdf,filename_uncP,cmd_uncompress,scanstyle)
     FILE **fileP;
     char *filename;
-    char **scanfilenameP;
+    char *filename_raw;
+    char **filename_dscP;
     char *cmd_scan_pdf;
+    char **filename_uncP;
+    char *cmd_uncompress;
+    int scanstyle;
 {
-    FILE *file = *fileP;
+    FILE *file;
     struct document *doc;
     int bb_set = NONE;
     int pages_set = NONE;
@@ -325,13 +385,85 @@ psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
     unsigned int section_len;	/* Place to accumulate the section length */
     char *next_char;		/* 1st char after text returned by gettext() */
     char *cp;
-    struct documentmedia *dmp;
+    Media dmp;
     FileData fd;
+    int respect_eof;            /* Derived from the scanstyle argument.
+                                   If set to 0 EOF comments will be ignored,
+                                   if set to 1 they will be taken seriously.
+                                   Purpose; Out there are many documents which 
+                                   include other DSC conforming documents without
+                                   without enclosing them by 'BeginDocument' and
+                                   'EndDocument' comments. This may cause fake EOF 
+				   comments to appear in the body of a page.
+				   Similarly, if respect_eof is set to false
+				   'Trailer' comments are ignored except of the
+				   last one found in the document.
+				*/
+    int ignore_dsc;             /* Derived from scanstyle.
+				   If set the document structure will be ignored.
+				*/
 
     BEGINMESSAGE(psscan)
 
-    fd =  ps_io_init(file);
+    if (cmd_uncompress) {
+      char b[2];
+      if (!(fread(b, sizeof(char),2, *fileP) == 2)
+	  || b[0] != '\037' || (b[1] != '\235' && b[1] != '\213')) {
+        rewind(*fileP);
+	cmd_uncompress=NULL;
+      }
+    }
+    if (cmd_uncompress) {
+      struct document *retval = NULL;
+      FILE *tmpfile = (FILE*)NULL;
+      char *filename_unc;
+      char cmd[512];
+      char s[512];
+      filename_unc=file_getTmpFilename(NULL,filename_raw);
+      sprintf(cmd,cmd_uncompress,filename,filename_unc);
+      INFMESSAGE(is compressed)
+      INFSMESSAGE(uncompress command,cmd)
+      if (ps_system(cmd) || file_fileIsNotUseful(filename_unc)) {
+	INFMESSAGE(uncompressing failed)
+unc_exec_failed:
+	sprintf(s,"Execution of\n%s\nfailed.",cmd);
+#if 0
+unc_failed:
+#endif
+	NotePopupShowMessage(s);
+	if (tmpfile) fclose(tmpfile);
+	unlink(filename_unc);
+unc_ok:
+	GV_XtFree(filename_unc);
+	ENDMESSAGE(psscan)
+        return(retval);
+      }
+      tmpfile = fopen(filename_unc, "r");
+      if (!tmpfile) goto unc_exec_failed;
+      fclose(*fileP);
+      *fileP = tmpfile;
+      retval = psscan(fileP,filename_unc,filename_raw,filename_dscP,cmd_scan_pdf,NULL,NULL,scanstyle);
+#if 0
+      if (!retval) {
+	sprintf(s,"333 Scanning\n%s\nfailed.",filename_unc);
+	goto unc_failed;
+      }
+#endif
+      *filename_uncP = (char*)GV_XtNewString(filename_unc);
+      goto unc_ok;
+    }
 
+    respect_eof = (scanstyle & SCANSTYLE_IGNORE_EOF) ? 0 : 1;
+    ignore_dsc = (scanstyle & SCANSTYLE_IGNORE_DSC) ? 1 : 0;
+    file = *fileP;
+
+    if (ignore_dsc) {
+      INFMESSAGE(ignoring DSC)
+      ENDMESSAGE(psscan)
+      return(NULL);
+    }
+
+    fd =  ps_io_init(file);
     if (!readline(fd,&line, &position, &line_len)) {
 	fprintf(stderr, "Warning: empty file.\n");
         ENDMESSAGE(psscan)
@@ -342,70 +474,60 @@ psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
     /* Header comments */
 
     if (line_len>1 && (iscomment(line,"%!PS-Adobe-") || iscomment(line + 1,"%!PS-Adobe-"))) {
-        INFMESSAGE(found "PS-Adobe-" comment)
-	doc = (struct document *) PS_malloc(sizeof(struct document));
-        CHECK_MALLOCED(doc);
-	memset(doc, 0, sizeof(struct document));
-	sscanf(line, "%*s %s", text);
-        /*###jp###*/
-	/*doc->epsf = iscomment(text, "EPSF-");*/
-	doc->epsf = iscomment(text, "EPSF");
-	doc->beginheader = position;
-	section_len = line_len;
-     } else if (iscomment(line,"%PDF-") && cmd_scan_pdf) {
-        char *scanfilename;
-        char cmd[512];
-        scanfilename=file_getTmpFilename(NULL,filename);
-        sprintf(cmd,cmd_scan_pdf,filename,scanfilename);
-        INFSMESSAGE(scan command,cmd)
+      INFMESSAGE(found "PS-Adobe-" comment)
+      doc = (struct document *) PS_malloc(sizeof(struct document));
+      CHECK_MALLOCED(doc);
+      memset(doc, 0, sizeof(struct document));
+      sscanf(line, "%*s %s", text);
+      /*###jp###*/
+      /*doc->epsf = iscomment(text, "EPSF-");*/
+      doc->epsf = iscomment(text, "EPSF");
+      doc->beginheader = position;
+      section_len = line_len;
+    } else if (iscomment(line,"%PDF-") && cmd_scan_pdf) {
+      struct document *retval = NULL;
+      FILE *tmpfile = (FILE*)NULL;
+      char *filename_dsc;
+      char cmd[512];
+      char s[512];
+      filename_dsc=file_getTmpFilename(NULL,filename_raw);
+      sprintf(cmd,cmd_scan_pdf,filename,filename_dsc);
+      INFMESSAGE(is PDF)
+      INFSMESSAGE(scan command,cmd)
 #ifdef VMS
- 	if ((system(cmd)&7)==1 && !file_fileIsNotUseful(scanfilename)) {
+      if ((system(cmd)&7)!=1 || file_fileIsNotUseful(filename_dsc)) {
 #else
- 	if (!system(cmd) && !file_fileIsNotUseful(scanfilename)) {
+      if (system(cmd) || file_fileIsNotUseful(filename_dsc)) {
 #endif
-           FILE *tmpfile = fopen(scanfilename, "r");
- 	   if (tmpfile) {
- 	      struct document *retval;
-              INFMESSAGE(is PDF)
-              fclose(*fileP);
- 	      *fileP = tmpfile;
- 	      retval = psscan(fileP,scanfilename,scanfilenameP,cmd_scan_pdf);
-              if (retval) {
-                 *scanfilenameP = (char*)PS_XtNewString(scanfilename);
-              } else {
-                 char s[512];
-                 sprintf(s,"Scanning\n%s\nfailed.",scanfilename);
-                 NotePopupShowMessage(s);
-              }
-              ps_io_exit(fd);
-              GV_XtFree(scanfilename);
-   	      return retval;
- 	   } else {
-              char s[512];
-              sprintf(s,"Execution of\n%s\nfailed.",cmd);
-              NotePopupShowMessage(s);
-              INFMESSAGE(cannot open scanfile) ENDMESSAGE(psscan)
-              fclose(tmpfile);
-              unlink(scanfilename);
-              GV_XtFree(scanfilename);
-              ps_io_exit(fd);
- 	      return NULL;
-           }
- 	} else {
-           char s[512];
-           sprintf(s,"Execution of\n%s\nfailed.",cmd);
-           NotePopupShowMessage(s);
-           unlink(scanfilename);
-           GV_XtFree(scanfilename);
-           INFMESSAGE(scan subprocess failed) ENDMESSAGE(psscan)
-           ps_io_exit(fd);
- 	   return NULL;
-        }
-    } else {
-        INFMESSAGE(unable to classify document)
-        ENDMESSAGE(psscan)
+	INFMESSAGE(scan subprocess failed)
+scan_exec_failed:
+	sprintf(s,"Execution of\n%s\nfailed.",cmd);
+scan_failed:
+	NotePopupShowMessage(s);
+	if (tmpfile) fclose(tmpfile);
+	unlink(filename_dsc);
+scan_ok:
+	GV_XtFree(filename_dsc);
         ps_io_exit(fd);
-	return(NULL);
+	ENDMESSAGE(psscan)
+        return(retval);
+      }
+      tmpfile = fopen(filename_dsc, "r");
+      if (!tmpfile) goto scan_exec_failed;
+      fclose(*fileP);
+      *fileP = tmpfile;
+      retval = psscan(fileP,filename_dsc,filename_raw,filename_dscP,cmd_scan_pdf,NULL,NULL,scanstyle);
+      if (!retval) {
+	sprintf(s,"Scanning\n%s\nfailed.",filename_dsc);
+	goto scan_failed;
+      }
+      *filename_dscP = (char*)GV_XtNewString(filename_dsc);
+      goto scan_ok;
+    } else {
+      INFMESSAGE(unable to classify document)
+      ENDMESSAGE(psscan)
+      ps_io_exit(fd);
+      return(NULL);
     }
     
     preread = 0;
@@ -511,8 +633,8 @@ psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
 	} else if (doc->nummedia == NONE &&
 		   iscomment(line+2, "DocumentMedia:")) {
 	    float w, h;
-	    doc->media = (struct documentmedia *)
-			 PS_malloc(sizeof (struct documentmedia));
+	    doc->media = (Media)
+			 PS_malloc(sizeof (MediaStruct));
             CHECK_MALLOCED(doc->media);
 	    doc->media[0].name = gettext(line+length("%%DocumentMedia:"),
 					 &next_char);
@@ -530,10 +652,10 @@ psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
 	    while (readline(fd, &line, &position, &line_len) &&
 		   DSCcomment(line) && iscomment(line+2, "+")) {
 		section_len += line_len;
-		doc->media = (struct documentmedia *)
+		doc->media = (Media)
 			     PS_realloc(doc->media,
 				     (doc->nummedia+1)*
-				     sizeof (struct documentmedia));
+				     sizeof (MediaStruct));
                 CHECK_MALLOCED(doc->media);
 		doc->media[doc->nummedia].name = gettext(line+length("%%+"),
 							 &next_char);
@@ -553,15 +675,16 @@ psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
 	} else if (doc->nummedia == NONE &&
 		   iscomment(line+2, "DocumentPaperSizes:")) {
 
-	    doc->media = (struct documentmedia *)
-			 PS_malloc(sizeof (struct documentmedia));
+	    doc->media = (Media)
+			 PS_malloc(sizeof (MediaStruct));
             CHECK_MALLOCED(doc->media);
 	    doc->media[0].name = gettext(line+length("%%DocumentPaperSizes:"),
 					 &next_char);
 	    if (doc->media[0].name != NULL) {
 		doc->media[0].width = 0;
 		doc->media[0].height = 0;
-		for (dmp=papersizes; dmp->name != NULL; dmp++) {
+		for (i=0; gv_medias[i]; i++) {
+ 		    dmp = gv_medias[i];
 		    /* Note: Paper size comment uses down cased paper size
 		     * name.  Case insensitive compares are only used for
 		     * PaperSize comments.
@@ -582,15 +705,16 @@ psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
 		    PS_free(doc->media[0].name);
 	    }
 	    while ((cp = gettext(next_char, &next_char))) {
-		doc->media = (struct documentmedia *)
+		doc->media = (Media)
 			     PS_realloc(doc->media,
 				     (doc->nummedia+1)*
-				     sizeof (struct documentmedia));
+				     sizeof (MediaStruct));
                 CHECK_MALLOCED(doc->media);
 		doc->media[doc->nummedia].name = cp;
 		doc->media[doc->nummedia].width = 0;
 		doc->media[doc->nummedia].height = 0;
-		for (dmp=papersizes; dmp->name != NULL; dmp++) {
+		for (i=0; gv_medias[i]; i++) {
+		   dmp = gv_medias[i];
 		    /* Note: Paper size comment uses down cased paper size
 		     * name.  Case insensitive compares are only used for
 		     * PaperSize comments.
@@ -602,7 +726,6 @@ psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
 				(char *)PS_malloc(strlen(dmp->name)+1);
                         CHECK_MALLOCED(doc->media[doc->nummedia].name);
 			strcpy(doc->media[doc->nummedia].name, dmp->name);
-			doc->media[doc->nummedia].name = dmp->name;
 			doc->media[doc->nummedia].width = dmp->width;
 			doc->media[doc->nummedia].height = dmp->height;
 			break;
@@ -619,15 +742,16 @@ psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
 		section_len += line_len;
 		next_char = line + length("%%+");
 		while ((cp = gettext(next_char, &next_char))) {
-		    doc->media = (struct documentmedia *)
+		    doc->media = (Media)
 				 PS_realloc(doc->media,
 					 (doc->nummedia+1)*
-					 sizeof (struct documentmedia));
+					 sizeof (MediaStruct));
                     CHECK_MALLOCED(doc->media);
 		    doc->media[doc->nummedia].name = cp;
 		    doc->media[doc->nummedia].width = 0;
 		    doc->media[doc->nummedia].height = 0;
-		    for (dmp=papersizes; dmp->name != NULL; dmp++) {
+		    for (i=0; gv_medias[i]; i++) {
+    		        dmp = gv_medias[i];
 			/* Note: Paper size comment uses down cased paper size
 			 * name.  Case insensitive compares are only used for
 			 * PaperSize comments.
@@ -804,7 +928,7 @@ psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
     if (!(DSCcomment(line) &&
 	  (iscomment(line+2, "Page:") ||
 	   iscomment(line+2, "Trailer") ||
-	   iscomment(line+2, "EOF")))) {
+           (respect_eof && iscomment(line+2, "EOF"))))) {
 	doc->beginsetup = beginsection;
 	beginsection = 0;
 	preread = 1;
@@ -814,7 +938,7 @@ psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
 	         (iscomment(line+2, "EndSetup") ||
 	          iscomment(line+2, "Page:") ||
 	          iscomment(line+2, "Trailer") ||
-	          iscomment(line+2, "EOF")))) {
+	          (respect_eof && iscomment(line+2, "EOF"))))) {
 	    if (!preread) section_len += line_len;
 	    preread = 0;
 	    if (!DSCcomment(line)) {
@@ -892,10 +1016,11 @@ psscan(fileP,filename,scanfilenameP,cmd_scan_pdf)
 	      (iscomment(line+2, "EndSetup") ||
 	      (iscomment(line+2, "Page:") ||
 	       iscomment(line+2, "Trailer") ||
-	       iscomment(line+2, "EOF")))) &&
+	       (respect_eof && iscomment(line+2, "EOF"))))) &&
              (readline(fd, &line, &position, &line_len))) {
+        section_len += line_len;
+        doc->lensetup = section_len - line_len;
 	doc->endsetup = position;
-	doc->lensetup += line_len;
       }
     }
     /* END Windows NT fix ###jp##*/
@@ -951,7 +1076,7 @@ continuepage:
 	       !(DSCcomment(line) &&
 	         (iscomment(line+2, "Page:") ||
 	          iscomment(line+2, "Trailer") ||
-	          iscomment(line+2, "EOF")))) {
+	          (respect_eof && iscomment(line+2, "EOF"))))) {
 	    section_len += line_len;
 	    if (!DSCcomment(line)) {
 		/* Do nothing */
@@ -1045,7 +1170,7 @@ continuepage:
     preread = 1;
     while ((preread ||
 	    readline(fd, &line, &position, &line_len)) &&
-	   !(DSCcomment(line) && iscomment(line+2, "EOF"))) {
+ 	   !(respect_eof && DSCcomment(line) && iscomment(line+2, "EOF"))) {
 	if (!preread) section_len += line_len;
 	preread = 0;
 	if (!DSCcomment(line)) {
@@ -1068,6 +1193,16 @@ continuepage:
 		}
 		goto newpage;
 	    }
+	} else if (!respect_eof && iscomment(line+2, "Trailer")) {
+	    /* What we thought was the start of the trailer was really */
+	    /* the trailer of an EPS on the page. */
+	    /* Set the end of the page to this trailer and keep scanning. */
+	    if (doc->numpages > 0) {
+		doc->pages[ doc->numpages-1 ].end = position;
+		doc->pages[ doc->numpages-1 ].len += section_len - line_len;
+	    }
+	    doc->begintrailer = position;
+	    section_len = line_len;
 	} else if (bb_set == ATEND && iscomment(line+2, "BoundingBox:")) {
 	    if (sscanf(line+length("%%BoundingBox:"), "%d %d %d %d",
 		       &(doc->boundingbox[LLX]),
@@ -1193,7 +1328,7 @@ psfree(doc)
 
 /*----------------------------------------------------------*/
 /*
- * gettextine -- skip over white space and return the rest of the line.
+ * gettextline -- skip over white space and return the rest of the line.
  *               If the text begins with '(' return the text string
  *		 using gettext().
  */
@@ -1332,8 +1467,9 @@ gettext(line, next_char)
 #define FD_STATUS_BUFTOOLARGE 1
 #define FD_STATUS_NOMORECHARS 2
 
-#define LINE_CHUNK_SIZE     512
-#define MAX_PS_IO_FGETCHARS_BUF_SIZE 131072
+#define LINE_CHUNK_SIZE     4096
+#define MAX_PS_IO_FGETCHARS_BUF_SIZE 57344
+#define BREAK_PS_IO_FGETCHARS_BUF_SIZE 49152
 
 static FileData ps_io_init(file)
    FILE *file;
@@ -1350,7 +1486,7 @@ static FileData ps_io_init(file)
    FD_FILE      = file;
    FD_FILE_DESC = fileno(file);
    FD_FILEPOS   = ftell(file);
-   FD_BUF_SIZE  = LINE_CHUNK_SIZE+1;
+   FD_BUF_SIZE  = (2*LINE_CHUNK_SIZE)+1;
    FD_BUF       = PS_XtMalloc(FD_BUF_SIZE);
    FD_BUF[0]    = '\0';
    ENDMESSAGE(ps_io_init)
@@ -1408,11 +1544,24 @@ ps_io_ftell(fd)
 /* ps_io_fgetchars */
 /*----------------------------------------------------------*/
 
+#ifdef USE_MEMMOVE_CODE
+static void ps_memmove (d, s, l)
+  char *d;
+  const char *s;
+  unsigned l;
+{
+  if (s < d) for (s += l, d += l; l; --l) *--d = *--s;
+  else if (s != d) for (; l; --l)         *d++ = *s++;
+}
+#else
+#   define ps_memmove memmove
+#endif
+
 static char * ps_io_fgetchars(fd,num)
    FileData fd;
    int num;
 {
-   char *eol=NULL;
+   char *eol=NULL,*tmp;
    size_t size_of_char = sizeof(char);
 
    BEGINMESSAGE(ps_io_fgetchars)
@@ -1442,7 +1591,13 @@ static char * ps_io_fgetchars(fd,num)
    do {
       if (num<0) { /* reading whole line */
          if (FD_BUF_END-FD_LINE_END) {
-            eol = strpbrk(FD_BUF+FD_LINE_END,"\n\r");
+ 	    /* strpbrk is faster but fails on lines with embedded NULLs 
+              eol = strpbrk(FD_BUF+FD_LINE_END,"\n\r");
+            */
+	    tmp = FD_BUF + FD_BUF_END;
+	    eol = FD_BUF + FD_LINE_END;
+	    while (eol < tmp && *eol != '\n' && *eol != '\r') eol++;
+	    if (eol >= tmp) eol = NULL;
             if (eol) {
                if (*eol=='\r' && *(eol+1)=='\n') eol += 2;
                else eol++;
@@ -1458,22 +1613,33 @@ static char * ps_io_fgetchars(fd,num)
 
       INFMESSAGE(no end of line yet)
 
+      if (FD_BUF_END - FD_LINE_BEGIN > BREAK_PS_IO_FGETCHARS_BUF_SIZE) {
+	INFMESSAGE(breaking line artificially)
+	eol = FD_BUF + FD_BUF_END - 1;
+	break;
+      }
+
       while (FD_BUF_SIZE < FD_BUF_END+LINE_CHUNK_SIZE+1) {
          if (FD_BUF_SIZE > MAX_PS_IO_FGETCHARS_BUF_SIZE) {
-            INFMESSAGE(buffer became too large)
+	   /* we should never get here, since the line is broken
+             artificially after BREAK_PS_IO_FGETCHARS_BUF_SIZE bytes. */
+            INFMESSAGE(buffer became to large)
             ENDMESSAGE(ps_io_fgetchars)
-            FD_STATUS      = FD_STATUS_BUFTOOLARGE;
-            return(NULL);
+	    fprintf(stderr, "gv: ps_io_fgetchars: Fatal Error: buffer became too large.\n");
+	    exit(-1);
          }
          if (FD_LINE_BEGIN) {
             INFMESSAGE(moving line to begin of buffer)
-            memmove((void*)FD_BUF,(void*)(FD_BUF+FD_LINE_BEGIN),
+            ps_memmove((void*)FD_BUF,(void*)(FD_BUF+FD_LINE_BEGIN),
                     ((size_t)(FD_BUF_END-FD_LINE_BEGIN+1))*size_of_char);
             FD_BUF_END    -= FD_LINE_BEGIN; 
             FD_LINE_BEGIN  = 0;
          } else {
             INFMESSAGE(enlarging buffer)
-            FD_BUF_SIZE    = FD_BUF_END+LINE_CHUNK_SIZE+1;
+	    /*
+              FD_BUF_SIZE    = FD_BUF_END+LINE_CHUNK_SIZE+1;
+	    */
+            FD_BUF_SIZE    = FD_BUF_SIZE+LINE_CHUNK_SIZE+1;
             IMESSAGE(FD_BUF_SIZE)
             FD_BUF         = PS_XtRealloc(FD_BUF,FD_BUF_SIZE);
          }
@@ -1485,7 +1651,7 @@ static char * ps_io_fgetchars(fd,num)
       if (num<0) FD_BUF_END += read(FD_FILE_DESC,FD_BUF+FD_BUF_END,LINE_CHUNK_SIZE);
       else       FD_BUF_END += fread(FD_BUF+FD_BUF_END,size_of_char,LINE_CHUNK_SIZE,FD_FILE);
 #else
-      /* read() seems to fail sometimes (????) so we always use fread ###jp###,07/31/96*/
+      /* read() seems to fail sometimes (? ? ?) so we always use fread ###jp###,07/31/96*/
       FD_BUF_END += fread(FD_BUF+FD_BUF_END,size_of_char,LINE_CHUNK_SIZE,FD_FILE);
 #endif
 
@@ -1522,9 +1688,11 @@ static char * ps_io_fgetchars(fd,num)
 #endif /* USE_FTELL_FOR_FILEPOS */
       FD_FILEPOS        += FD_LINE_LEN;
 
+#if 0
    SMESSAGE(FD_BUF+FD_LINE_BEGIN)
    IIMESSAGE(FD_LINE_BEGIN,FD_LINE_END)
    IIMESSAGE(FD_BUF_END,FD_LINE_LEN)
+#endif
 
    ENDMESSAGE(ps_io_fgetchars)
    return(FD_BUF+FD_LINE_BEGIN);
@@ -1584,6 +1752,26 @@ static char * readline (fd, lineP, positionP, line_lenP)
            INFMESSAGE(completed skipping until comment1 or comment2)\
         }
 
+#if 0
+   if ((scanstyle&SCANSTYLE_MISSING_BEGINDOCUMENT) &&
+       (line[0] == '%') &&
+       (*line_lenP > 11) &&
+       (iscomment(line,"%!PS-Adobe-") || iscomment(line + 1,"%!PS-Adobe-"))) {
+     char *c=line+11;
+     while (*c && !isspace(*c)) c++;
+     if (isspace(*c)) while (*c && isspace(*c)) c++;
+     /* don't skip EPSF files */
+       printf("line in question: %s\n",line);
+     if (strncmp(c,"EPSF",4)) {
+       printf("skipping starts here: %s\n",line);
+       SKIP_UNTIL_1("EOF")
+       *line_lenP += nbytes;
+       readline(fd, &line, NULL, &nbytes);
+       printf("skipping ends here: %s\n",line);
+     }
+   }
+   else
+#endif
    if  (!IS_COMMENT("Begin"))     {} /* Do nothing */
    else if IS_BEGIN("Document:")  SKIP_UNTIL_1("EndDocument")
    else if IS_BEGIN("Feature:")   SKIP_UNTIL_1("EndFeature")
