@@ -2,7 +2,7 @@
 **
 ** save.c
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -37,7 +37,8 @@
 #include INC_X11(Intrinsic.h)
 #include "Ghostview.h"
 
-#include "gv.h"
+#include "types.h"
+#include "config.h"
 #include "d_memdebug.h"
 #include "error.h"
 #include "file.h"
@@ -111,16 +112,43 @@ print_file(print_command,print_filename)
    String print_filename;
 {
    String error=NULL;
-   char buf[BUFSIZ];
+   char *c,*p;
+   Cardinal m,n;
    String printfail=GV_ERROR_PRINT_FAIL;
-   
+
    BEGINMESSAGE(print_file)
-   sprintf(buf, "%s %s",print_command,print_filename);
-   INFSMESSAGE(printing:,buf)
-   if (SYSTEM_FAILED_ON(buf)) {
-      sprintf(buf,printfail,print_command); 
-      error = GV_XtNewString(buf);
+
+   p = GV_XtNewString(print_command);
+   n=0;
+   c=p;
+   while ((c=strstr(c,"%s"))) { c+=2; n++; }
+   m = (strlen(p)+(n>0?n:1)*strlen(print_filename)+5)*sizeof(char);
+   c = (char*) GV_XtMalloc(m);
+   if (n>0) {
+     char *e,*s;
+     e=s=p;
+     c[0]='\0';
+     while (s) {
+       s=strstr(e,"%s");
+       if (s) *s='\0';
+       strcat(c,e);
+       if (s) {
+	 strcat(c,print_filename);
+         e=s+2;
+       } 
+       else s=NULL;
+     }
+   } else {
+     sprintf(c, "%s %s",p,print_filename);
    }
+   INFSMESSAGE(printing:,c)
+   if (SYSTEM_FAILED_ON(c)) {
+     m = (strlen(printfail)+strlen(c)+1)*sizeof(char);
+       error = (char*) GV_XtMalloc(m);
+     sprintf(error,printfail,c); 
+   }
+   GV_XtFree(c);
+   GV_XtFree(p);
    ENDMESSAGE(print_file)
    return(error);
 }
@@ -189,10 +217,11 @@ save_forkPDFToPSConversion(sd)
 /*------------------------------------------------------------*/
 
 static String
-save_copyToFile(save_filename,src_filename,pagelist)
+save_copyToFile(save_filename,src_filename,pagelist,scanstyle)
    String save_filename;
    String src_filename;
    char *pagelist;
+   int scanstyle;
 {
    FILE *save_file=NULL;
    FILE *src_file=NULL;
@@ -209,8 +238,10 @@ save_copyToFile(save_filename,src_filename,pagelist)
    if (!error) {
       if (pagelist) {
          Document src_doc=NULL;
+	 String s = GV_XtNewString(src_filename);
+	 s = file_getUsefulName(s);
          INFMESSAGE(scanning document)
-         doc_scanFile(&src_file,&src_doc,src_filename,NULL,NULL);
+         doc_scanFile(&src_file,&src_doc,src_filename,s,NULL,NULL,NULL,NULL,scanstyle);
          if (src_doc) {
             INFMESSAGE(calling pscopydoc)
             pscopydoc(save_file,src_filename,src_doc,pagelist);
@@ -245,7 +276,7 @@ save_saveFile(sd)
    SaveData sd;
 {
    char *src_fn;
-   String error=NULL;
+   String error=NULL,s;
 
    BEGINMESSAGE(save_saveFile)
 
@@ -256,7 +287,12 @@ save_saveFile(sd)
    }
 
    if (!error && sd->convert && !sd->conv_fn) {
-      sd->conv_fn = file_getTmpFilename(NULL,sd->save_fn);
+      s = sd->save_fn ? sd->save_fn : sd->src_fn;
+      s = GV_XtNewString(s);
+      s = file_getUsefulName(s);
+      s = file_pdfname2psname(s);
+      sd->conv_fn = file_getTmpFilename(NULL,s);
+      GV_XtFree(s);
       INFSMESSAGE(converting from file,sd->src_fn)
       INFSMESSAGE(converting to file,sd->conv_fn)
       error = save_forkPDFToPSConversion(sd);
@@ -268,10 +304,16 @@ save_saveFile(sd)
    if (sd->conv_fn) src_fn = sd->conv_fn;
 
    if (!error && sd->save_to_file) {
-      if (!sd->save_fn) sd->save_fn = file_getTmpFilename(NULL,NULL);
+      if (!sd->save_fn) {
+	s = GV_XtNewString(sd->src_fn);
+	s = file_getUsefulName(s);
+	s = file_pdfname2psname(s);      
+	sd->save_fn = file_getTmpFilename(NULL,s);
+	GV_XtFree(s);
+      }
       INFSMESSAGE(saving from file,src_fn)
       INFSMESSAGE(saving to file,sd->save_fn)
-      error = save_copyToFile(sd->save_fn,src_fn,sd->pagelist);
+      error = save_copyToFile(sd->save_fn,src_fn,sd->pagelist,sd->scanstyle);
       src_fn = sd->save_fn;
    }
 
