@@ -2,7 +2,7 @@
 **
 ** versionp.h
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physicss
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -35,12 +35,17 @@
 #define  VERSION_APPLIC_CONTEXT	app_con  			/* the Application Context */
 
 char *versionIdentification[] = {
-        "gv 2.9.4",
-        "March 1997",
+        "gv 3.5.8",
+        "June 1997",
 	0
 };
 
-char *versionCompatibility = "gv 2.9.0";
+/*
+   declare it writable due to problems with sscanf() in main_versionIsCompatible() 
+   on HP/UX 9.07
+   [suggested by Christian Illinger (illinger@lepsi.in2p3.fr)]
+*/
+char versionCompatibility[] = "gv 3.5.2";
                                             
 char * copyright = "\
  gv -- An X11 user interface for GhostScript. \n\
