@@ -2,7 +2,7 @@
 **
 ** zoom.c
 **
-** Copyright (C) 1995-1997 Johannes Plass
+** Copyright (C) 1995, 1996, 1997 Johannes Plass
 ** 
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 ** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ** 
 ** Author:   Johannes Plass (plass@thep.physik.uni-mainz.de)
-**           Department of Physic
+**           Department of Physics
 **           Johannes Gutenberg-University
 **           Mainz, Germany
 **
@@ -43,19 +43,21 @@
 #include INC_X11(StringDefs.h)
 #include INC_X11(Shell.h)
 #include INC_XAW(Cardinals.h)
-#include INC_XAW(Command.h)
 #include "Aaa.h"
+#include "Button.h"
 #include "Clip.h"
 #include "Frame.h"
 #include "Ghostview.h"
 
+#include "types.h"
+#include "config.h"
 #include "callbacks.h"
 #include "ps.h"
 #include "doc_misc.h"
-#include "gv.h"
 #include "main_resources.h"
 #include "main_globals.h"
-#include "popup_misc.h"
+#include "popup.h"
+#include "widgets_misc.h"
 #include "zoom.h"
 
 static char* zoom_popup      = "zoomPopup";
@@ -80,11 +82,7 @@ zoom_createZoom(w, call_data)
     Cardinal n;
     struct stat sbuf;
     GhostviewReturnStruct *p = (GhostviewReturnStruct *)call_data;
-#if 0
-    XtPageOrientation orient;
-    Dimension width, height;
-    Widget zredisplay;
-#endif
+    Boolean b;
     int llx;
     int lly;
     int urx;
@@ -101,7 +99,7 @@ zoom_createZoom(w, call_data)
 
     BEGINMESSAGE(zoom_createZoom)
 
-    if (!psfile) {
+    if (!gv_psfile) {
        INFMESSAGE(no file) ENDMESSAGE(zoom_createZoom)
        return;
     }
@@ -110,12 +108,12 @@ zoom_createZoom(w, call_data)
        INFMESSAGE1(file has changed) ENDMESSAGE1(zoom_createZoom)return;
     }
 
-    filename = (gv_scanfilename ? gv_scanfilename : gv_filename);
+    filename = (gv_filename_dsc ? gv_filename_dsc : (gv_filename_unc ? gv_filename_unc : gv_filename));
 
-    llx = p->psx - p->width/2;
-    lly = p->psy - p->height/2;
-    urx = p->psx + p->width/2;
-    ury = p->psy + p->height/2;
+    llx = p->psx;
+    lly = p->psy;
+    urx = p->psx + p->width  - 1; if (urx<llx) urx=llx;
+    ury = p->psy + p->height - 1; if (ury<lly) ury=lly;
     IIMESSAGE(llx,lly) IIMESSAGE(urx,ury)
 
     /* Make sure zoom window doesn't go off the edge of the page */
@@ -137,15 +135,11 @@ zoom_createZoom(w, call_data)
     zshell = XtCreatePopupShell(zoom_popup, topLevelShellWidgetClass,toplevel,args,n);
 
                                				       n=0;
-       XtSetArg(args[n], XtNresizeWidth, True);	               n++;
-       XtSetArg(args[n], XtNresizeHeight, True);	       n++;
-       XtSetArg(args[n], XtNconditionedResize, False);	       n++;
        XtSetArg(args[n], XtNminimumWidth, 70);			n++;
        XtSetArg(args[n], XtNminimumHeight,70);			n++; 
     zcontrol = XtCreateManagedWidget(zoom_control,aaaWidgetClass,zshell,args,n);
 
-									n=0;
-       XtSetArg(args[n], XtNresizable, True);			n++;
+								n=0;
    zviewFrame = XtCreateManagedWidget("zviewFrame", frameWidgetClass,zcontrol,args,n);
 
 									n=0;
@@ -153,15 +147,11 @@ zoom_createZoom(w, call_data)
        XtAddCallback(zviewClip, XtNreportCallback,cb_pageAdjustNotify,(XtPointer)NULL);
 
 									n=0;
-           XtSetArg(args[n], XtNresizeWidth, True);	               n++;
-           XtSetArg(args[n], XtNresizeHeight, True);	       n++;
-           XtSetArg(args[n], XtNconditionedResize, False);	       n++;
    zviewControl = XtCreateManagedWidget("zviewControl", aaaWidgetClass,zviewClip,args,n);
 
    xto     = doc_convDocOrientToXtOrient(gv_orientation, gv_swap_landscape );
 
 							       n=0;
-       XtSetArg(args[n], XtNresizable, True);		       n++;
        XtSetArg(args[n], XtNorientation,xto);                  n++;
        XtSetArg(args[n], XtNllx, llx);      	               n++;
        XtSetArg(args[n], XtNlly, lly);      	               n++;
@@ -174,14 +164,13 @@ zoom_createZoom(w, call_data)
        XtSetArg(args[n], XtNbottomMargin, bottom_margin);      n++;
        XtSetArg(args[n], XtNlxdpi, (1000*p->xdpi));            n++;
        XtSetArg(args[n], XtNlydpi, (1000*p->ydpi));            n++;
-       XtSetArg(args[n], XtNautoSetup,False);		       n++;
        XtSetArg(args[n], XtNinterpreter,gv_gs_interpreter);    n++;
-       if (gv_gs_safer)  XtSetArg(args[n], XtNsafer,True);
-       else              XtSetArg(args[n], XtNsafer,False);
-                                                               n++;
-       if (gv_gs_quiet)  XtSetArg(args[n], XtNquiet,True);
-       else              XtSetArg(args[n], XtNquiet,False);
-                                                               n++;
+       b = gv_gs_safer ? True : False;
+       XtSetArg(args[n], XtNsafer,b);                          n++;
+       b = gv_gs_quiet ? True : False;
+       XtSetArg(args[n], XtNquiet,b);                          n++;
+       b = app_res.use_bpixmap ? True : False;
+       XtSetArg(args[n], XtNuseBackingPixmap,b);               n++;
        XtSetArg(args[n], XtNarguments,gv_gs_arguments);        n++;
        if (!toc_text) {
           XtSetArg(args[n], XtNfilename, filename);	       n++;
@@ -193,24 +182,10 @@ zoom_createZoom(w, call_data)
     XtAddCallback(zpage, XtNdestroyCallback, cb_destroyGhost,(XtPointer)zpage); 
 
 							    n=0;
-    zdismiss = XtCreateManagedWidget(zoom_dismiss, commandWidgetClass,zcontrol, args, n);
+    zdismiss = XtCreateManagedWidget(zoom_dismiss, buttonWidgetClass,zcontrol, args, n);
     XtAddCallback(zdismiss, XtNcallback,cb_destroy,(XtPointer)zshell);
 
     XtRealizeWidget(zshell);
-#if 0
-    {
-       Dimension w,h;
-       Position x,y;
-                                                               n=0;
-       XtSetArg(args[n], XtNx,  &x);     n++;
-       XtSetArg(args[n], XtNy, &y);    n++;
-       XtSetArg(args[n], XtNwidth,  &w);     n++;
-       XtSetArg(args[n], XtNheight, &h);    n++;
-       XtGetValues(zviewControl, args, n);
-       INFIIMESSAGE(#### zviewControl,x,y)
-       INFIIMESSAGE(#### zviewControl,w,h)
-    }
-#endif
     {
        Dimension page_width,page_height;
        INFMESSAGE(adjusting size)
@@ -224,14 +199,14 @@ zoom_createZoom(w, call_data)
        XtSetArg(args[n], XtNheight, page_height);              n++;
        XtSetValues(zpage, args, n);
     }
+    XtInstallAccelerators(zcontrol,zdismiss);
+    widgets_preferButton(zdismiss,1);
 
     {
       GhostviewReturnStruct p;
       INFMESSAGE(positioning zoom popup)
       IIMESSAGE(llx,lly) IIMESSAGE(urx,ury)
-      GhostviewGetAreaOfBB(page,
-                        (Position)(llx),(Position)(lly),
-                        (Position)(urx),(Position)(ury),&p);
+      GhostviewGetAreaOfBB(page,llx,lly,urx,ury,&p);
       popup_positionPopup(
           zshell,page,POPUP_POSITION_POS_CENTER,
           ((int)(p.psx)+(int)(p.width/2)),
@@ -272,7 +247,7 @@ zoom_createZoom(w, call_data)
 
 #if 0
 							    n=0;
-    zredisplay = XtCreateManagedWidget(zoom_redisplay, commandWidgetClass,zcontrol, args, n);
+    zredisplay = XtCreateManagedWidget(zoom_redisplay, buttonWidgetClass,zcontrol, args, n);
     if (toc_text) {
        INFMESSAGE(have useful toc)
        XtAddCallback(zredisplay, XtNcallback, cb_redisplay, (XtPointer)current_page);
